type token =
  | CODE of (Coqpp_ast.code)
  | COMMENT of (string)
  | IDENT of (string)
  | QUALID of (string)
  | STRING of (string)
  | INT of (int)
  | VERNAC
  | TACTIC
  | GRAMMAR
  | DOC_GRAMMAR
  | EXTEND
  | END
  | DECLARE
  | PLUGIN
  | DEPRECATED
  | ARGUMENT
  | RAW_PRINTED
  | GLOB_PRINTED
  | COMMAND
  | CLASSIFIED
  | STATE
  | PRINTED
  | TYPED
  | INTERPRETED
  | GLOBALIZED
  | SUBSTITUTED
  | BY
  | AS
  | BANGBRACKET
  | HASHBRACKET
  | LBRACKET
  | RBRACKET
  | PIPE
  | ARROW
  | FUN
  | COMMA
  | EQUAL
  | STAR
  | LPAREN
  | RPAREN
  | COLON
  | SEMICOLON
  | GLOBAL
  | FIRST
  | LAST
  | BEFORE
  | AFTER
  | LEVEL
  | LEFTA
  | RIGHTA
  | NONA
  | EOF

open Parsing;;
let _ = parse_error;;
# 12 "coqpp/coqpp_parse.mly"

open Coqpp_ast

let starts s pat =
  let len = String.length s in
  let patlen = String.length pat in
  if patlen <= len && String.sub s 0 patlen = pat then
    Some (String.sub s patlen (len - patlen))
  else None

let ends s pat =
  let len = String.length s in
  let patlen = String.length pat in
  if patlen <= len && String.sub s (len - patlen) patlen = pat then
    Some (String.sub s 0 (len - patlen))
  else None

let between s pat1 pat2 = match starts s pat1 with
| None -> None
| Some s -> ends s pat2  

let without_sep k sep r =
  if sep <> "" then raise Parsing.Parse_error else k r

let parse_user_entry s sep =
  let table = [
    "ne_", "_list", without_sep (fun r -> Ulist1 r);
    "ne_", "_list_sep", (fun sep r -> Ulist1sep (r, sep));
    "", "_list", without_sep (fun r -> Ulist0 r);
    "", "_list_sep", (fun sep r -> Ulist0sep (r, sep));
    "", "_opt", without_sep (fun r -> Uopt r);
  ] in
  let rec parse s sep = function
  | [] ->
    let () = without_sep ignore sep () in
    begin match starts s "tactic" with
    | Some ("0"|"1"|"2"|"3"|"4"|"5" as s) -> Uentryl ("tactic", int_of_string s)
    | Some _ | None -> Uentry s
    end
  | (pat1, pat2, k) :: rem ->
    match between s pat1 pat2 with
    | None -> parse s sep rem
    | Some s ->
      let r = parse s "" table in
      k sep r      
  in
  parse s sep table

let no_code = { code = ""; loc = { loc_start=Lexing.dummy_pos; loc_end=Lexing.dummy_pos} }

# 109 "coqpp/coqpp_parse.ml"
let yytransl_const = [|
  263 (* VERNAC *);
  264 (* TACTIC *);
  265 (* GRAMMAR *);
  266 (* DOC_GRAMMAR *);
  267 (* EXTEND *);
  268 (* END *);
  269 (* DECLARE *);
  270 (* PLUGIN *);
  271 (* DEPRECATED *);
  272 (* ARGUMENT *);
  273 (* RAW_PRINTED *);
  274 (* GLOB_PRINTED *);
  275 (* COMMAND *);
  276 (* CLASSIFIED *);
  277 (* STATE *);
  278 (* PRINTED *);
  279 (* TYPED *);
  280 (* INTERPRETED *);
  281 (* GLOBALIZED *);
  282 (* SUBSTITUTED *);
  283 (* BY *);
  284 (* AS *);
  285 (* BANGBRACKET *);
  286 (* HASHBRACKET *);
  287 (* LBRACKET *);
  288 (* RBRACKET *);
  289 (* PIPE *);
  290 (* ARROW *);
  291 (* FUN *);
  292 (* COMMA *);
  293 (* EQUAL *);
  294 (* STAR *);
  295 (* LPAREN *);
  296 (* RPAREN *);
  297 (* COLON *);
  298 (* SEMICOLON *);
  299 (* GLOBAL *);
  300 (* FIRST *);
  301 (* LAST *);
  302 (* BEFORE *);
  303 (* AFTER *);
  304 (* LEVEL *);
  305 (* LEFTA *);
  306 (* RIGHTA *);
  307 (* NONA *);
    0 (* EOF *);
    0|]

let yytransl_block = [|
  257 (* CODE *);
  258 (* COMMENT *);
  259 (* IDENT *);
  260 (* QUALID *);
  261 (* STRING *);
  262 (* INT *);
    0|]

let yylhs = "\255\255\
\001\000\002\000\002\000\003\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\004\000\005\000\008\000\008\000\014\000\
\014\000\018\000\018\000\019\000\019\000\015\000\015\000\016\000\
\016\000\017\000\017\000\013\000\013\000\021\000\021\000\021\000\
\006\000\022\000\022\000\023\000\023\000\023\000\024\000\024\000\
\025\000\025\000\026\000\028\000\028\000\027\000\027\000\032\000\
\032\000\032\000\033\000\033\000\030\000\030\000\031\000\031\000\
\007\000\034\000\034\000\035\000\035\000\020\000\020\000\036\000\
\029\000\029\000\037\000\037\000\037\000\037\000\010\000\010\000\
\011\000\011\000\038\000\038\000\012\000\012\000\039\000\040\000\
\040\000\042\000\042\000\042\000\042\000\042\000\043\000\043\000\
\044\000\044\000\045\000\045\000\045\000\041\000\041\000\046\000\
\047\000\047\000\048\000\048\000\049\000\050\000\050\000\051\000\
\051\000\052\000\052\000\054\000\054\000\054\000\054\000\054\000\
\053\000\053\000\009\000\055\000\055\000\056\000\056\000\057\000\
\057\000\058\000\059\000\059\000\059\000\060\000\060\000\061\000\
\061\000\062\000\062\000\063\000\063\000\063\000\063\000\000\000"

let yylen = "\002\000\
\002\000\000\000\002\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\003\000\006\000\012\000\007\000\000\000\
\003\000\000\000\003\000\000\000\003\000\000\000\003\000\000\000\
\003\000\000\000\003\000\000\000\003\000\001\000\002\000\005\000\
\008\000\001\000\001\000\000\000\003\000\003\000\000\000\002\000\
\001\000\002\000\010\000\000\000\003\000\000\000\003\000\001\000\
\002\000\003\000\003\000\001\000\000\000\001\000\000\000\002\000\
\007\000\000\000\002\000\000\000\002\000\000\000\002\000\006\000\
\000\000\002\000\001\000\001\000\004\000\006\000\001\000\001\000\
\000\000\004\000\000\000\002\000\000\000\002\000\007\000\000\000\
\001\000\001\000\001\000\002\000\002\000\002\000\000\000\001\000\
\000\000\001\000\001\000\001\000\001\000\001\000\003\000\005\000\
\000\000\001\000\001\000\003\000\003\000\000\000\001\000\001\000\
\003\000\003\000\001\000\001\000\003\000\003\000\003\000\001\000\
\001\000\002\000\002\000\000\000\002\000\006\000\004\000\001\000\
\003\000\001\000\000\000\001\000\002\000\001\000\003\000\003\000\
\001\000\001\000\002\000\001\000\003\000\003\000\001\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\004\000\005\000\000\000\000\000\000\000\000\000\
\000\000\000\000\136\000\000\000\000\000\006\000\007\000\008\000\
\009\000\010\000\011\000\035\000\000\000\034\000\000\000\000\000\
\000\000\072\000\071\000\000\000\115\000\000\000\000\000\000\000\
\001\000\003\000\000\000\000\000\000\000\000\000\000\000\117\000\
\012\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\059\000\
\000\000\000\000\000\000\000\000\000\000\000\000\119\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\061\000\000\000\000\000\000\000\000\000\013\000\
\078\000\000\000\135\000\000\000\000\000\132\000\000\000\000\000\
\122\000\000\000\126\000\129\000\000\000\030\000\000\000\000\000\
\000\000\000\000\017\000\000\000\015\000\063\000\037\000\038\000\
\040\000\000\000\000\000\000\000\057\000\076\000\074\000\082\000\
\083\000\000\000\000\000\000\000\000\000\081\000\000\000\000\000\
\000\000\118\000\000\000\000\000\131\000\000\000\031\000\000\000\
\000\000\000\000\000\000\067\000\000\000\000\000\000\000\000\000\
\033\000\042\000\084\000\085\000\086\000\000\000\128\000\134\000\
\133\000\121\000\127\000\000\000\023\000\000\000\000\000\000\000\
\000\000\000\000\066\000\000\000\000\000\000\000\000\000\000\000\
\088\000\000\000\000\000\000\000\000\000\025\000\000\000\000\000\
\000\000\000\000\000\000\000\000\047\000\000\000\000\000\000\000\
\000\000\091\000\092\000\093\000\000\000\090\000\000\000\032\000\
\027\000\000\000\000\000\000\000\000\000\069\000\064\000\051\000\
\050\000\045\000\000\000\079\000\000\000\095\000\019\000\000\000\
\000\000\000\000\000\000\000\000\112\000\000\000\000\000\000\000\
\000\000\098\000\000\000\000\000\103\000\000\000\107\000\000\000\
\021\000\014\000\070\000\054\000\000\000\000\000\000\000\000\000\
\000\000\096\000\000\000\000\000\000\000\114\000\000\000\000\000\
\106\000\111\000\110\000\109\000\100\000\101\000\105\000\056\000\
\000\000\043\000"

let yydgoto = "\002\000\
\011\000\012\000\013\000\014\000\015\000\016\000\017\000\018\000\
\019\000\086\000\048\000\061\000\051\000\053\000\098\000\130\000\
\152\000\169\000\188\000\069\000\096\000\023\000\055\000\074\000\
\107\000\108\000\136\000\160\000\133\000\221\000\232\000\157\000\
\158\000\046\000\058\000\070\000\134\000\078\000\062\000\117\000\
\162\000\118\000\163\000\181\000\182\000\164\000\209\000\210\000\
\211\000\212\000\213\000\214\000\215\000\216\000\029\000\030\000\
\087\000\088\000\089\000\090\000\091\000\092\000\093\000"

let yysindex = "\015\000\
\161\255\000\000\000\000\000\000\061\255\016\255\024\255\182\255\
\007\255\047\255\000\000\085\000\161\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\089\255\000\000\092\255\115\255\
\182\255\000\000\000\000\101\255\000\000\182\255\148\255\154\255\
\000\000\000\000\194\255\201\255\160\255\140\255\176\255\000\000\
\000\000\188\255\190\255\193\255\214\255\174\255\175\255\182\255\
\155\255\189\255\190\255\191\255\187\255\168\255\202\255\000\000\
\218\255\187\255\182\255\184\255\215\255\182\255\000\000\026\255\
\020\255\204\255\225\255\198\255\219\255\187\255\229\255\230\255\
\231\255\199\255\000\000\223\255\182\255\195\255\146\255\000\000\
\000\000\203\255\000\000\026\255\063\255\000\000\206\255\208\255\
\000\000\197\255\000\000\000\000\063\255\000\000\020\255\233\255\
\216\255\217\255\000\000\087\255\000\000\000\000\000\000\000\000\
\000\000\220\255\232\255\199\255\000\000\000\000\000\000\000\000\
\000\000\240\255\241\255\242\255\221\255\000\000\063\255\222\255\
\209\255\000\000\026\255\026\255\000\000\015\255\000\000\250\255\
\226\255\234\255\224\255\000\000\236\255\087\255\182\255\227\255\
\000\000\000\000\000\000\000\000\000\000\243\255\000\000\000\000\
\000\000\000\000\000\000\020\255\000\000\254\255\235\255\244\255\
\010\000\237\255\000\000\238\255\245\255\239\255\013\000\249\255\
\000\000\246\255\129\255\251\255\009\255\000\000\017\000\252\255\
\002\000\046\255\025\000\182\255\000\000\182\255\253\255\087\255\
\255\255\000\000\000\000\000\000\003\000\000\000\243\255\000\000\
\000\000\026\000\004\000\187\255\027\000\000\000\000\000\000\000\
\000\000\000\000\005\000\000\000\067\255\000\000\000\000\029\000\
\021\000\006\000\020\000\007\000\000\000\067\255\070\255\008\000\
\011\000\000\000\009\000\014\000\000\000\012\000\000\000\070\255\
\000\000\000\000\000\000\000\000\016\000\070\255\018\000\019\000\
\031\000\000\000\067\255\037\000\067\255\000\000\038\000\023\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\039\000\000\000"

let yyrindex = "\000\000\
\001\001\000\000\000\000\000\000\000\000\000\000\000\000\001\000\
\000\000\000\000\000\000\000\000\001\001\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\001\000\000\000\000\000\
\000\000\000\000\000\000\000\000\031\255\169\255\000\000\000\000\
\000\000\071\255\066\255\255\254\000\000\075\255\000\000\033\000\
\000\000\000\000\112\255\000\000\035\000\000\000\022\000\000\000\
\000\000\035\000\024\000\000\000\000\000\033\000\000\000\170\255\
\000\000\122\255\000\000\000\000\000\000\035\000\000\000\000\000\
\000\000\000\000\000\000\000\000\024\000\000\000\030\000\000\000\
\000\000\006\255\000\000\170\255\000\000\000\000\000\000\028\000\
\000\000\173\255\000\000\000\000\083\255\000\000\000\000\095\255\
\000\000\132\255\000\000\032\000\000\000\000\000\000\000\000\000\
\000\000\125\255\000\000\040\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\170\255\177\255\000\000\000\000\000\000\000\000\
\000\000\134\255\010\255\000\000\000\000\032\000\000\000\034\000\
\000\000\000\000\000\000\000\000\000\000\110\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\093\255\
\000\000\000\000\000\000\049\255\000\000\036\000\000\000\000\000\
\000\000\000\000\041\000\042\000\000\000\000\000\000\000\000\000\
\102\255\000\000\000\000\000\000\000\000\043\000\000\000\032\000\
\000\000\000\000\000\000\000\000\000\000\000\000\110\255\000\000\
\000\000\000\000\000\000\035\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\099\255\000\000\000\000\000\000\
\000\000\000\000\166\255\002\255\000\000\044\000\000\000\021\255\
\000\000\000\000\045\000\000\000\000\000\046\000\000\000\142\255\
\000\000\000\000\000\000\000\000\047\000\000\000\000\000\000\000\
\000\000\000\000\044\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000"

let yygindex = "\000\000\
\000\000\036\001\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\248\255\000\000\247\000\000\000\007\001\000\000\000\000\
\000\000\000\000\000\000\200\255\164\255\000\000\000\000\000\000\
\210\000\000\000\000\000\000\000\126\255\000\000\000\000\145\000\
\000\000\000\000\000\000\000\000\000\000\246\000\000\000\000\000\
\142\000\000\000\000\000\000\000\000\000\000\000\000\000\172\255\
\000\000\000\000\097\000\000\000\068\255\000\000\041\001\000\000\
\180\255\000\000\000\000\000\000\205\000\247\255\000\000"

let yytablesize = 337
let yytable = "\028\000\
\116\000\076\000\126\000\155\000\072\000\072\000\072\000\120\000\
\072\000\072\000\072\000\127\000\068\000\102\000\068\000\001\000\
\038\000\127\000\224\000\036\000\031\000\028\000\094\000\108\000\
\108\000\108\000\024\000\230\000\082\000\027\000\083\000\036\000\
\072\000\233\000\025\000\072\000\072\000\072\000\072\000\060\000\
\072\000\068\000\058\000\072\000\072\000\195\000\146\000\072\000\
\184\000\072\000\077\000\108\000\148\000\060\000\108\000\165\000\
\084\000\032\000\095\000\108\000\108\000\020\000\108\000\058\000\
\085\000\026\000\027\000\083\000\077\000\204\000\027\000\205\000\
\026\000\027\000\205\000\121\000\021\000\016\000\058\000\022\000\
\052\000\189\000\028\000\125\000\033\000\190\000\060\000\028\000\
\028\000\131\000\052\000\132\000\028\000\084\000\028\000\028\000\
\028\000\206\000\016\000\035\000\206\000\085\000\036\000\028\000\
\018\000\207\000\029\000\060\000\207\000\143\000\018\000\029\000\
\029\000\020\000\130\000\130\000\029\000\037\000\029\000\029\000\
\029\000\223\000\130\000\016\000\130\000\018\000\156\000\029\000\
\016\000\016\000\097\000\201\000\102\000\022\000\020\000\016\000\
\016\000\016\000\022\000\022\000\087\000\039\000\237\000\024\000\
\016\000\026\000\022\000\022\000\024\000\024\000\026\000\026\000\
\041\000\046\000\022\000\046\000\042\000\024\000\087\000\087\000\
\087\000\003\000\004\000\192\000\024\000\156\000\026\000\005\000\
\006\000\007\000\008\000\073\000\073\000\009\000\045\000\113\000\
\010\000\178\000\179\000\180\000\073\000\113\000\047\000\113\000\
\026\000\027\000\063\000\064\000\208\000\112\000\113\000\114\000\
\115\000\116\000\071\000\072\000\043\000\208\000\208\000\053\000\
\053\000\123\000\123\000\044\000\124\000\124\000\049\000\208\000\
\125\000\125\000\050\000\052\000\054\000\208\000\056\000\059\000\
\065\000\067\000\208\000\068\000\208\000\057\000\073\000\075\000\
\079\000\099\000\080\000\097\000\100\000\103\000\101\000\106\000\
\104\000\105\000\109\000\127\000\111\000\122\000\124\000\119\000\
\123\000\129\000\128\000\137\000\139\000\140\000\141\000\161\000\
\145\000\135\000\149\000\142\000\150\000\144\000\166\000\159\000\
\002\000\116\000\116\000\151\000\168\000\167\000\153\000\116\000\
\116\000\116\000\116\000\154\000\170\000\116\000\171\000\175\000\
\116\000\185\000\172\000\187\000\173\000\177\000\186\000\176\000\
\174\000\191\000\199\000\183\000\194\000\217\000\200\000\202\000\
\218\000\197\000\220\000\236\000\203\000\238\000\240\000\242\000\
\196\000\227\000\226\000\222\000\077\000\219\000\062\000\228\000\
\034\000\234\000\231\000\041\000\081\000\229\000\039\000\225\000\
\241\000\066\000\235\000\120\000\080\000\138\000\193\000\065\000\
\044\000\075\000\110\000\048\000\198\000\239\000\040\000\089\000\
\147\000\094\000\049\000\000\000\099\000\102\000\000\000\104\000\
\055\000"

let yycheck = "\008\000\
\000\000\058\000\095\000\134\000\003\001\004\001\005\001\084\000\
\003\001\004\001\005\001\003\001\003\001\070\000\005\001\001\000\
\025\000\003\001\207\000\021\001\014\001\030\000\003\001\003\001\
\004\001\005\001\011\001\216\000\003\001\004\001\005\001\033\001\
\031\001\222\000\011\001\034\001\031\001\032\001\033\001\048\000\
\039\001\032\001\012\001\042\001\039\001\176\000\123\000\042\001\
\040\001\048\001\059\000\031\001\038\001\062\000\034\001\148\000\
\031\001\011\001\039\001\039\001\040\001\001\001\042\001\033\001\
\039\001\003\001\004\001\005\001\077\000\003\001\004\001\005\001\
\003\001\004\001\005\001\085\000\016\001\012\001\048\001\019\001\
\032\001\036\001\012\001\093\000\000\000\040\001\012\001\017\001\
\018\001\003\001\042\001\005\001\022\001\031\001\024\001\025\001\
\026\001\031\001\033\001\011\001\031\001\039\001\011\001\033\001\
\012\001\039\001\012\001\033\001\039\001\119\000\018\001\017\001\
\018\001\012\001\032\001\033\001\022\001\003\001\024\001\025\001\
\026\001\206\000\040\001\012\001\042\001\033\001\135\000\033\001\
\017\001\018\001\032\001\188\000\034\001\012\001\033\001\024\001\
\025\001\026\001\017\001\018\001\031\001\041\001\227\000\012\001\
\033\001\012\001\025\001\026\001\017\001\018\001\017\001\018\001\
\005\001\029\001\033\001\031\001\003\001\026\001\049\001\050\001\
\051\001\001\001\002\001\172\000\033\001\174\000\033\001\007\001\
\008\001\009\001\010\001\003\001\004\001\013\001\015\001\034\001\
\016\001\049\001\050\001\051\001\012\001\040\001\043\001\042\001\
\003\001\004\001\032\001\033\001\197\000\044\001\045\001\046\001\
\047\001\048\001\027\001\028\001\003\001\206\000\207\000\034\001\
\035\001\032\001\033\001\003\001\032\001\033\001\031\001\216\000\
\032\001\033\001\023\001\022\001\020\001\222\000\001\001\041\001\
\028\001\027\001\227\000\033\001\229\000\048\001\021\001\006\001\
\041\001\001\001\012\001\024\001\031\001\001\001\012\001\033\001\
\003\001\003\001\012\001\003\001\042\001\032\001\042\001\037\001\
\033\001\025\001\027\001\012\001\005\001\005\001\005\001\005\001\
\040\001\030\001\001\001\031\001\027\001\032\001\001\001\029\001\
\000\000\001\001\002\001\026\001\017\001\027\001\039\001\007\001\
\008\001\009\001\010\001\032\001\003\001\013\001\034\001\003\001\
\016\001\001\001\037\001\018\001\032\001\032\001\027\001\031\001\
\042\001\001\001\001\001\033\001\032\001\001\001\027\001\005\001\
\012\001\031\001\015\001\005\001\032\001\001\001\001\001\001\001\
\042\001\033\001\032\001\037\001\012\001\040\001\012\001\034\001\
\013\000\032\001\035\001\012\001\062\000\042\001\033\001\048\001\
\034\001\051\000\040\001\032\001\031\001\108\000\174\000\032\001\
\031\001\042\001\077\000\032\001\183\000\229\000\030\000\031\001\
\124\000\032\001\032\001\255\255\032\001\034\001\255\255\034\001\
\034\001"

let yynames_const = "\
  VERNAC\000\
  TACTIC\000\
  GRAMMAR\000\
  DOC_GRAMMAR\000\
  EXTEND\000\
  END\000\
  DECLARE\000\
  PLUGIN\000\
  DEPRECATED\000\
  ARGUMENT\000\
  RAW_PRINTED\000\
  GLOB_PRINTED\000\
  COMMAND\000\
  CLASSIFIED\000\
  STATE\000\
  PRINTED\000\
  TYPED\000\
  INTERPRETED\000\
  GLOBALIZED\000\
  SUBSTITUTED\000\
  BY\000\
  AS\000\
  BANGBRACKET\000\
  HASHBRACKET\000\
  LBRACKET\000\
  RBRACKET\000\
  PIPE\000\
  ARROW\000\
  FUN\000\
  COMMA\000\
  EQUAL\000\
  STAR\000\
  LPAREN\000\
  RPAREN\000\
  COLON\000\
  SEMICOLON\000\
  GLOBAL\000\
  FIRST\000\
  LAST\000\
  BEFORE\000\
  AFTER\000\
  LEVEL\000\
  LEFTA\000\
  RIGHTA\000\
  NONA\000\
  EOF\000\
  "

let yynames_block = "\
  CODE\000\
  COMMENT\000\
  IDENT\000\
  QUALID\000\
  STRING\000\
  INT\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'nodes) in
    Obj.repr(
# 84 "coqpp/coqpp_parse.mly"
  ( _1 )
# 481 "coqpp/coqpp_parse.ml"
               : Coqpp_ast.t))
; (fun __caml_parser_env ->
    Obj.repr(
# 89 "coqpp/coqpp_parse.mly"
  ( [] )
# 487 "coqpp/coqpp_parse.ml"
               : 'nodes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'node) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'nodes) in
    Obj.repr(
# 91 "coqpp/coqpp_parse.mly"
  ( _1 :: _2 )
# 495 "coqpp/coqpp_parse.ml"
               : 'nodes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 95 "coqpp/coqpp_parse.mly"
       ( Code _1 )
# 502 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 96 "coqpp/coqpp_parse.mly"
          ( Comment _1 )
# 509 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'declare_plugin) in
    Obj.repr(
# 97 "coqpp/coqpp_parse.mly"
                 ( _1 )
# 516 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'grammar_extend) in
    Obj.repr(
# 98 "coqpp/coqpp_parse.mly"
                 ( _1 )
# 523 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'vernac_extend) in
    Obj.repr(
# 99 "coqpp/coqpp_parse.mly"
                ( _1 )
# 530 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'tactic_extend) in
    Obj.repr(
# 100 "coqpp/coqpp_parse.mly"
                ( _1 )
# 537 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'argument_extend) in
    Obj.repr(
# 101 "coqpp/coqpp_parse.mly"
                  ( _1 )
# 544 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram) in
    Obj.repr(
# 102 "coqpp/coqpp_parse.mly"
           ( _1 )
# 551 "coqpp/coqpp_parse.ml"
               : 'node))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 106 "coqpp/coqpp_parse.mly"
                        ( DeclarePlugin _3 )
# 558 "coqpp/coqpp_parse.ml"
               : 'declare_plugin))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'qualid_or_ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'globals) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'gram_entries) in
    Obj.repr(
# 111 "coqpp/coqpp_parse.mly"
  ( GramExt { gramext_name = _3; gramext_globals = _4; gramext_entries = _5 } )
# 567 "coqpp/coqpp_parse.ml"
               : 'grammar_extend))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 9 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 8 : 'typed_opt) in
    let _5 = (Parsing.peek_val __caml_parser_env 7 : 'printed_opt) in
    let _6 = (Parsing.peek_val __caml_parser_env 6 : 'interpreted_opt) in
    let _7 = (Parsing.peek_val __caml_parser_env 5 : 'globalized_opt) in
    let _8 = (Parsing.peek_val __caml_parser_env 4 : 'substituted_opt) in
    let _9 = (Parsing.peek_val __caml_parser_env 3 : 'raw_printed_opt) in
    let _10 = (Parsing.peek_val __caml_parser_env 2 : 'glob_printed_opt) in
    let _11 = (Parsing.peek_val __caml_parser_env 1 : 'tactic_rules) in
    Obj.repr(
# 125 "coqpp/coqpp_parse.mly"
  ( ArgumentExt {
    argext_name = _3;
    argext_rules = _11;
    argext_rprinter = _9;
    argext_gprinter = _10;
    argext_tprinter = _5;
    argext_interp = _6;
    argext_glob = _7;
    argext_subst = _8;
    argext_type = _4;
  } )
# 592 "coqpp/coqpp_parse.ml"
               : 'argument_extend))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'printed_opt) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'tactic_rules) in
    Obj.repr(
# 137 "coqpp/coqpp_parse.mly"
  ( VernacArgumentExt {
    vernacargext_name = _4;
    vernacargext_printer = _5;
    vernacargext_rules = _6;
  } )
# 605 "coqpp/coqpp_parse.ml"
               : 'argument_extend))
; (fun __caml_parser_env ->
    Obj.repr(
# 145 "coqpp/coqpp_parse.mly"
  ( None )
# 611 "coqpp/coqpp_parse.ml"
               : 'printed_opt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 146 "coqpp/coqpp_parse.mly"
                  ( Some _3 )
# 618 "coqpp/coqpp_parse.ml"
               : 'printed_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 150 "coqpp/coqpp_parse.mly"
  ( None )
# 624 "coqpp/coqpp_parse.ml"
               : 'raw_printed_opt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 151 "coqpp/coqpp_parse.mly"
                      ( Some _3 )
# 631 "coqpp/coqpp_parse.ml"
               : 'raw_printed_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 155 "coqpp/coqpp_parse.mly"
  ( None )
# 637 "coqpp/coqpp_parse.ml"
               : 'glob_printed_opt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 156 "coqpp/coqpp_parse.mly"
                       ( Some _3 )
# 644 "coqpp/coqpp_parse.ml"
               : 'glob_printed_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 160 "coqpp/coqpp_parse.mly"
  ( None )
# 650 "coqpp/coqpp_parse.ml"
               : 'interpreted_opt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 161 "coqpp/coqpp_parse.mly"
                      ( Some _3 )
# 657 "coqpp/coqpp_parse.ml"
               : 'interpreted_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 165 "coqpp/coqpp_parse.mly"
  ( None )
# 663 "coqpp/coqpp_parse.ml"
               : 'globalized_opt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 166 "coqpp/coqpp_parse.mly"
                     ( Some _3 )
# 670 "coqpp/coqpp_parse.ml"
               : 'globalized_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 170 "coqpp/coqpp_parse.mly"
  ( None )
# 676 "coqpp/coqpp_parse.ml"
               : 'substituted_opt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 171 "coqpp/coqpp_parse.mly"
                      ( Some _3 )
# 683 "coqpp/coqpp_parse.ml"
               : 'substituted_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 175 "coqpp/coqpp_parse.mly"
  ( None )
# 689 "coqpp/coqpp_parse.ml"
               : 'typed_opt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'argtype) in
    Obj.repr(
# 176 "coqpp/coqpp_parse.mly"
                   ( Some _3 )
# 696 "coqpp/coqpp_parse.ml"
               : 'typed_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 180 "coqpp/coqpp_parse.mly"
        ( ExtraArgType _1 )
# 703 "coqpp/coqpp_parse.ml"
               : 'argtype))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'argtype) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 181 "coqpp/coqpp_parse.mly"
                (
    match _2 with
    | "list" -> ListArgType _1
    | "option" ->  OptArgType _1
    | _ -> raise Parsing.Parse_error
  )
# 716 "coqpp/coqpp_parse.ml"
               : 'argtype))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'argtype) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'argtype) in
    Obj.repr(
# 187 "coqpp/coqpp_parse.mly"
                                     ( PairArgType (_2, _4) )
# 724 "coqpp/coqpp_parse.ml"
               : 'argtype))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 6 : 'vernac_entry) in
    let _4 = (Parsing.peek_val __caml_parser_env 4 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 3 : 'vernac_classifier) in
    let _6 = (Parsing.peek_val __caml_parser_env 2 : 'vernac_state) in
    let _7 = (Parsing.peek_val __caml_parser_env 1 : 'vernac_rules) in
    Obj.repr(
# 192 "coqpp/coqpp_parse.mly"
  ( VernacExt {
    vernacext_name = _4;
    vernacext_entry = _2;
    vernacext_class = _5;
    vernacext_state = _6;
    vernacext_rules = _7;
  } )
# 741 "coqpp/coqpp_parse.ml"
               : 'vernac_extend))
; (fun __caml_parser_env ->
    Obj.repr(
# 202 "coqpp/coqpp_parse.mly"
          ( None )
# 747 "coqpp/coqpp_parse.ml"
               : 'vernac_entry))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 203 "coqpp/coqpp_parse.mly"
       ( Some _1 )
# 754 "coqpp/coqpp_parse.ml"
               : 'vernac_entry))
; (fun __caml_parser_env ->
    Obj.repr(
# 207 "coqpp/coqpp_parse.mly"
  ( ClassifDefault )
# 760 "coqpp/coqpp_parse.ml"
               : 'vernac_classifier))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 208 "coqpp/coqpp_parse.mly"
                     ( ClassifCode _3 )
# 767 "coqpp/coqpp_parse.ml"
               : 'vernac_classifier))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 209 "coqpp/coqpp_parse.mly"
                      ( ClassifName _3 )
# 774 "coqpp/coqpp_parse.ml"
               : 'vernac_classifier))
; (fun __caml_parser_env ->
    Obj.repr(
# 213 "coqpp/coqpp_parse.mly"
  ( None )
# 780 "coqpp/coqpp_parse.ml"
               : 'vernac_state))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 214 "coqpp/coqpp_parse.mly"
              ( Some _2 )
# 787 "coqpp/coqpp_parse.ml"
               : 'vernac_state))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'vernac_rule) in
    Obj.repr(
# 218 "coqpp/coqpp_parse.mly"
              ( [_1] )
# 794 "coqpp/coqpp_parse.ml"
               : 'vernac_rules))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'vernac_rule) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'vernac_rules) in
    Obj.repr(
# 219 "coqpp/coqpp_parse.mly"
                           ( _1 :: _2 )
# 802 "coqpp/coqpp_parse.ml"
               : 'vernac_rules))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 8 : 'vernac_attributes_opt) in
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'rule_state) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'ext_tokens) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'rule_deprecation) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'rule_classifier) in
    let _10 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 224 "coqpp/coqpp_parse.mly"
  ( {
      vernac_atts = _2;
      vernac_state = _3;
      vernac_toks = _5;
      vernac_depr = _7;
      vernac_class= _8;
      vernac_body = _10;
  } )
# 821 "coqpp/coqpp_parse.ml"
               : 'vernac_rule))
; (fun __caml_parser_env ->
    Obj.repr(
# 235 "coqpp/coqpp_parse.mly"
  ( None )
# 827 "coqpp/coqpp_parse.ml"
               : 'rule_state))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 236 "coqpp/coqpp_parse.mly"
                             ( Some _2 )
# 834 "coqpp/coqpp_parse.ml"
               : 'rule_state))
; (fun __caml_parser_env ->
    Obj.repr(
# 240 "coqpp/coqpp_parse.mly"
  ( None )
# 840 "coqpp/coqpp_parse.ml"
               : 'vernac_attributes_opt))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'vernac_attributes) in
    Obj.repr(
# 241 "coqpp/coqpp_parse.mly"
                                         ( Some _2 )
# 847 "coqpp/coqpp_parse.ml"
               : 'vernac_attributes_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'vernac_attribute) in
    Obj.repr(
# 245 "coqpp/coqpp_parse.mly"
                   ( [_1] )
# 854 "coqpp/coqpp_parse.ml"
               : 'vernac_attributes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'vernac_attribute) in
    Obj.repr(
# 246 "coqpp/coqpp_parse.mly"
                             ( [_1] )
# 861 "coqpp/coqpp_parse.ml"
               : 'vernac_attributes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'vernac_attribute) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'vernac_attributes) in
    Obj.repr(
# 247 "coqpp/coqpp_parse.mly"
                                               ( _1 :: _3 )
# 869 "coqpp/coqpp_parse.ml"
               : 'vernac_attributes))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'qualid_or_ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'qualid_or_ident) in
    Obj.repr(
# 251 "coqpp/coqpp_parse.mly"
                                        ( (_1, _3) )
# 877 "coqpp/coqpp_parse.ml"
               : 'vernac_attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'qualid_or_ident) in
    Obj.repr(
# 252 "coqpp/coqpp_parse.mly"
                  ( (_1, _1) )
# 884 "coqpp/coqpp_parse.ml"
               : 'vernac_attribute))
; (fun __caml_parser_env ->
    Obj.repr(
# 256 "coqpp/coqpp_parse.mly"
  ( false )
# 890 "coqpp/coqpp_parse.ml"
               : 'rule_deprecation))
; (fun __caml_parser_env ->
    Obj.repr(
# 257 "coqpp/coqpp_parse.mly"
             ( true )
# 896 "coqpp/coqpp_parse.ml"
               : 'rule_deprecation))
; (fun __caml_parser_env ->
    Obj.repr(
# 261 "coqpp/coqpp_parse.mly"
  ( None )
# 902 "coqpp/coqpp_parse.ml"
               : 'rule_classifier))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 262 "coqpp/coqpp_parse.mly"
           ( Some _2 )
# 909 "coqpp/coqpp_parse.ml"
               : 'rule_classifier))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 4 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'tactic_deprecated) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'tactic_level) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'tactic_rules) in
    Obj.repr(
# 267 "coqpp/coqpp_parse.mly"
  ( TacticExt { tacext_name = _3; tacext_deprecated = _4; tacext_level = _5; tacext_rules = _6 } )
# 919 "coqpp/coqpp_parse.ml"
               : 'tactic_extend))
; (fun __caml_parser_env ->
    Obj.repr(
# 271 "coqpp/coqpp_parse.mly"
  ( None )
# 925 "coqpp/coqpp_parse.ml"
               : 'tactic_deprecated))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 272 "coqpp/coqpp_parse.mly"
                  ( Some _2 )
# 932 "coqpp/coqpp_parse.ml"
               : 'tactic_deprecated))
; (fun __caml_parser_env ->
    Obj.repr(
# 276 "coqpp/coqpp_parse.mly"
  ( None )
# 938 "coqpp/coqpp_parse.ml"
               : 'tactic_level))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 277 "coqpp/coqpp_parse.mly"
            ( Some _2 )
# 945 "coqpp/coqpp_parse.ml"
               : 'tactic_level))
; (fun __caml_parser_env ->
    Obj.repr(
# 281 "coqpp/coqpp_parse.mly"
  ( [] )
# 951 "coqpp/coqpp_parse.ml"
               : 'tactic_rules))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'tactic_rule) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'tactic_rules) in
    Obj.repr(
# 282 "coqpp/coqpp_parse.mly"
                           ( _1 :: _2 )
# 959 "coqpp/coqpp_parse.ml"
               : 'tactic_rules))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'ext_tokens) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 287 "coqpp/coqpp_parse.mly"
  ( { tac_toks = _3; tac_body = _6 } )
# 967 "coqpp/coqpp_parse.ml"
               : 'tactic_rule))
; (fun __caml_parser_env ->
    Obj.repr(
# 291 "coqpp/coqpp_parse.mly"
  ( [] )
# 973 "coqpp/coqpp_parse.ml"
               : 'ext_tokens))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'ext_token) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'ext_tokens) in
    Obj.repr(
# 292 "coqpp/coqpp_parse.mly"
                       ( _1 :: _2 )
# 981 "coqpp/coqpp_parse.ml"
               : 'ext_tokens))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 296 "coqpp/coqpp_parse.mly"
         ( ExtTerminal _1 )
# 988 "coqpp/coqpp_parse.ml"
               : 'ext_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 297 "coqpp/coqpp_parse.mly"
        (
  let e = parse_user_entry _1 "" in
  ExtNonTerminal (e, TokNone) 
  )
# 998 "coqpp/coqpp_parse.ml"
               : 'ext_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 301 "coqpp/coqpp_parse.mly"
                            (
  let e = parse_user_entry _1 "" in
  ExtNonTerminal (e, TokName _3)
  )
# 1009 "coqpp/coqpp_parse.ml"
               : 'ext_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 305 "coqpp/coqpp_parse.mly"
                                         (
  let e = parse_user_entry _1 _5 in
  ExtNonTerminal (e, TokName _3)
)
# 1021 "coqpp/coqpp_parse.ml"
               : 'ext_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 312 "coqpp/coqpp_parse.mly"
         ( _1 )
# 1028 "coqpp/coqpp_parse.ml"
               : 'qualid_or_ident))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 313 "coqpp/coqpp_parse.mly"
        ( _1 )
# 1035 "coqpp/coqpp_parse.ml"
               : 'qualid_or_ident))
; (fun __caml_parser_env ->
    Obj.repr(
# 317 "coqpp/coqpp_parse.mly"
  ( [] )
# 1041 "coqpp/coqpp_parse.ml"
               : 'globals))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'idents) in
    Obj.repr(
# 318 "coqpp/coqpp_parse.mly"
                                ( _3 )
# 1048 "coqpp/coqpp_parse.ml"
               : 'globals))
; (fun __caml_parser_env ->
    Obj.repr(
# 322 "coqpp/coqpp_parse.mly"
  ( [] )
# 1054 "coqpp/coqpp_parse.ml"
               : 'idents))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'qualid_or_ident) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'idents) in
    Obj.repr(
# 323 "coqpp/coqpp_parse.mly"
                         ( _1 :: _2 )
# 1062 "coqpp/coqpp_parse.ml"
               : 'idents))
; (fun __caml_parser_env ->
    Obj.repr(
# 327 "coqpp/coqpp_parse.mly"
  ( [] )
# 1068 "coqpp/coqpp_parse.ml"
               : 'gram_entries))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'gram_entry) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'gram_entries) in
    Obj.repr(
# 328 "coqpp/coqpp_parse.mly"
                          ( _1 :: _2 )
# 1076 "coqpp/coqpp_parse.ml"
               : 'gram_entries))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 6 : 'qualid_or_ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 4 : 'position_opt) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'levels) in
    Obj.repr(
# 333 "coqpp/coqpp_parse.mly"
  ( { gentry_name = _1; gentry_pos = _3; gentry_rules = _5; } )
# 1085 "coqpp/coqpp_parse.ml"
               : 'gram_entry))
; (fun __caml_parser_env ->
    Obj.repr(
# 337 "coqpp/coqpp_parse.mly"
  ( None )
# 1091 "coqpp/coqpp_parse.ml"
               : 'position_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'position) in
    Obj.repr(
# 338 "coqpp/coqpp_parse.mly"
           ( Some _1 )
# 1098 "coqpp/coqpp_parse.ml"
               : 'position_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 342 "coqpp/coqpp_parse.mly"
        ( First )
# 1104 "coqpp/coqpp_parse.ml"
               : 'position))
; (fun __caml_parser_env ->
    Obj.repr(
# 343 "coqpp/coqpp_parse.mly"
       ( Last )
# 1110 "coqpp/coqpp_parse.ml"
               : 'position))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 344 "coqpp/coqpp_parse.mly"
                ( Before _2 )
# 1117 "coqpp/coqpp_parse.ml"
               : 'position))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 345 "coqpp/coqpp_parse.mly"
               ( After _2 )
# 1124 "coqpp/coqpp_parse.ml"
               : 'position))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 346 "coqpp/coqpp_parse.mly"
               ( Level _2 )
# 1131 "coqpp/coqpp_parse.ml"
               : 'position))
; (fun __caml_parser_env ->
    Obj.repr(
# 350 "coqpp/coqpp_parse.mly"
  ( None )
# 1137 "coqpp/coqpp_parse.ml"
               : 'string_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 351 "coqpp/coqpp_parse.mly"
         ( Some _1 )
# 1144 "coqpp/coqpp_parse.ml"
               : 'string_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 355 "coqpp/coqpp_parse.mly"
  ( None )
# 1150 "coqpp/coqpp_parse.ml"
               : 'assoc_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'assoc) in
    Obj.repr(
# 356 "coqpp/coqpp_parse.mly"
        ( Some _1 )
# 1157 "coqpp/coqpp_parse.ml"
               : 'assoc_opt))
; (fun __caml_parser_env ->
    Obj.repr(
# 360 "coqpp/coqpp_parse.mly"
        ( LeftA )
# 1163 "coqpp/coqpp_parse.ml"
               : 'assoc))
; (fun __caml_parser_env ->
    Obj.repr(
# 361 "coqpp/coqpp_parse.mly"
         ( RightA )
# 1169 "coqpp/coqpp_parse.ml"
               : 'assoc))
; (fun __caml_parser_env ->
    Obj.repr(
# 362 "coqpp/coqpp_parse.mly"
       ( NonA )
# 1175 "coqpp/coqpp_parse.ml"
               : 'assoc))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'level) in
    Obj.repr(
# 366 "coqpp/coqpp_parse.mly"
        ( [_1] )
# 1182 "coqpp/coqpp_parse.ml"
               : 'levels))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'level) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'levels) in
    Obj.repr(
# 367 "coqpp/coqpp_parse.mly"
                    ( _1 :: _3 )
# 1190 "coqpp/coqpp_parse.ml"
               : 'levels))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'string_opt) in
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'assoc_opt) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'rules_opt) in
    Obj.repr(
# 372 "coqpp/coqpp_parse.mly"
  ( { grule_label = _1; grule_assoc = _2; grule_prods = _4; } )
# 1199 "coqpp/coqpp_parse.ml"
               : 'level))
; (fun __caml_parser_env ->
    Obj.repr(
# 376 "coqpp/coqpp_parse.mly"
  ( [] )
# 1205 "coqpp/coqpp_parse.ml"
               : 'rules_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'rules) in
    Obj.repr(
# 377 "coqpp/coqpp_parse.mly"
        ( _1 )
# 1212 "coqpp/coqpp_parse.ml"
               : 'rules_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'rule) in
    Obj.repr(
# 381 "coqpp/coqpp_parse.mly"
       ( [_1] )
# 1219 "coqpp/coqpp_parse.ml"
               : 'rules))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'rule) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'rules) in
    Obj.repr(
# 382 "coqpp/coqpp_parse.mly"
                  ( _1 :: _3 )
# 1227 "coqpp/coqpp_parse.ml"
               : 'rules))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'symbols_opt) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Coqpp_ast.code) in
    Obj.repr(
# 387 "coqpp/coqpp_parse.mly"
  ( { gprod_symbs = _1; gprod_body = _3; } )
# 1235 "coqpp/coqpp_parse.ml"
               : 'rule))
; (fun __caml_parser_env ->
    Obj.repr(
# 391 "coqpp/coqpp_parse.mly"
  ( [] )
# 1241 "coqpp/coqpp_parse.ml"
               : 'symbols_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'symbols) in
    Obj.repr(
# 392 "coqpp/coqpp_parse.mly"
          ( _1 )
# 1248 "coqpp/coqpp_parse.ml"
               : 'symbols_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'symbol) in
    Obj.repr(
# 396 "coqpp/coqpp_parse.mly"
         ( [_1] )
# 1255 "coqpp/coqpp_parse.ml"
               : 'symbols))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'symbol) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'symbols) in
    Obj.repr(
# 397 "coqpp/coqpp_parse.mly"
                           ( _1 :: _3 )
# 1263 "coqpp/coqpp_parse.ml"
               : 'symbols))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gram_tokens) in
    Obj.repr(
# 401 "coqpp/coqpp_parse.mly"
                          ( (Some _1, _3) )
# 1271 "coqpp/coqpp_parse.ml"
               : 'symbol))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'gram_tokens) in
    Obj.repr(
# 402 "coqpp/coqpp_parse.mly"
              ( (None, _1) )
# 1278 "coqpp/coqpp_parse.ml"
               : 'symbol))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'qualid_or_ident) in
    Obj.repr(
# 406 "coqpp/coqpp_parse.mly"
                  ( GSymbQualid (_1, None) )
# 1285 "coqpp/coqpp_parse.ml"
               : 'gram_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'qualid_or_ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 407 "coqpp/coqpp_parse.mly"
                               ( GSymbQualid (_1, Some _3) )
# 1293 "coqpp/coqpp_parse.ml"
               : 'gram_token))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'gram_tokens) in
    Obj.repr(
# 408 "coqpp/coqpp_parse.mly"
                            ( GSymbParen _2 )
# 1300 "coqpp/coqpp_parse.ml"
               : 'gram_token))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'rules) in
    Obj.repr(
# 409 "coqpp/coqpp_parse.mly"
                          ( GSymbProd _2 )
# 1307 "coqpp/coqpp_parse.ml"
               : 'gram_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 410 "coqpp/coqpp_parse.mly"
         ( GSymbString _1 )
# 1314 "coqpp/coqpp_parse.ml"
               : 'gram_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'gram_token) in
    Obj.repr(
# 414 "coqpp/coqpp_parse.mly"
             ( [_1] )
# 1321 "coqpp/coqpp_parse.ml"
               : 'gram_tokens))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'gram_token) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'gram_tokens) in
    Obj.repr(
# 415 "coqpp/coqpp_parse.mly"
                         ( _1 :: _2 )
# 1329 "coqpp/coqpp_parse.ml"
               : 'gram_tokens))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_entries) in
    Obj.repr(
# 420 "coqpp/coqpp_parse.mly"
  ( GramExt { gramext_name = ""; gramext_globals=[]; gramext_entries = _2 } )
# 1336 "coqpp/coqpp_parse.ml"
               : 'doc_gram))
; (fun __caml_parser_env ->
    Obj.repr(
# 423 "coqpp/coqpp_parse.mly"
  ( [] )
# 1342 "coqpp/coqpp_parse.ml"
               : 'doc_gram_entries))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'doc_gram_entry) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_entries) in
    Obj.repr(
# 424 "coqpp/coqpp_parse.mly"
                                   ( _1 :: _2 )
# 1350 "coqpp/coqpp_parse.ml"
               : 'doc_gram_entries))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'qualid_or_ident) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'doc_gram_rules) in
    Obj.repr(
# 429 "coqpp/coqpp_parse.mly"
  ( { gentry_name = _1; gentry_pos = None;
      gentry_rules = [{ grule_label = None; grule_assoc = None; grule_prods = _5; }] } )
# 1359 "coqpp/coqpp_parse.ml"
               : 'doc_gram_entry))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'qualid_or_ident) in
    Obj.repr(
# 432 "coqpp/coqpp_parse.mly"
  ( { gentry_name = _1; gentry_pos = None;
      gentry_rules = [{ grule_label = None; grule_assoc = None; grule_prods = []; }] } )
# 1367 "coqpp/coqpp_parse.ml"
               : 'doc_gram_entry))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_rule) in
    Obj.repr(
# 437 "coqpp/coqpp_parse.mly"
                ( [_1] )
# 1374 "coqpp/coqpp_parse.ml"
               : 'doc_gram_rules))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'doc_gram_rule) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_rules) in
    Obj.repr(
# 438 "coqpp/coqpp_parse.mly"
                                    ( _1 :: _3 )
# 1382 "coqpp/coqpp_parse.ml"
               : 'doc_gram_rules))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_symbols_opt) in
    Obj.repr(
# 442 "coqpp/coqpp_parse.mly"
                       ( { gprod_symbs = _1; gprod_body = no_code; } )
# 1389 "coqpp/coqpp_parse.ml"
               : 'doc_gram_rule))
; (fun __caml_parser_env ->
    Obj.repr(
# 446 "coqpp/coqpp_parse.mly"
  ( [] )
# 1395 "coqpp/coqpp_parse.ml"
               : 'doc_gram_symbols_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_symbols) in
    Obj.repr(
# 447 "coqpp/coqpp_parse.mly"
                   ( _1 )
# 1402 "coqpp/coqpp_parse.ml"
               : 'doc_gram_symbols_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'doc_gram_symbols) in
    Obj.repr(
# 448 "coqpp/coqpp_parse.mly"
                             ( _1 )
# 1409 "coqpp/coqpp_parse.ml"
               : 'doc_gram_symbols_opt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_symbol) in
    Obj.repr(
# 452 "coqpp/coqpp_parse.mly"
                  ( [_1] )
# 1416 "coqpp/coqpp_parse.ml"
               : 'doc_gram_symbols))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'doc_gram_symbols) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_symbol) in
    Obj.repr(
# 453 "coqpp/coqpp_parse.mly"
                                             ( _1 @ [_3] )
# 1424 "coqpp/coqpp_parse.ml"
               : 'doc_gram_symbols))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_gram_tokens) in
    Obj.repr(
# 457 "coqpp/coqpp_parse.mly"
                                   ( (Some _1, _3) )
# 1432 "coqpp/coqpp_parse.ml"
               : 'doc_gram_symbol))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_gram_tokens) in
    Obj.repr(
# 458 "coqpp/coqpp_parse.mly"
                       ( (None, _1) )
# 1439 "coqpp/coqpp_parse.ml"
               : 'doc_gram_symbol))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_gram_token) in
    Obj.repr(
# 462 "coqpp/coqpp_parse.mly"
                      ( [_1] )
# 1446 "coqpp/coqpp_parse.ml"
               : 'doc_gram_gram_tokens))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'doc_gram_gram_token) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'doc_gram_gram_tokens) in
    Obj.repr(
# 463 "coqpp/coqpp_parse.mly"
                                           ( _1 :: _2 )
# 1454 "coqpp/coqpp_parse.ml"
               : 'doc_gram_gram_tokens))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'qualid_or_ident) in
    Obj.repr(
# 467 "coqpp/coqpp_parse.mly"
                  ( GSymbQualid (_1, None) )
# 1461 "coqpp/coqpp_parse.ml"
               : 'doc_gram_gram_token))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'doc_gram_gram_tokens) in
    Obj.repr(
# 468 "coqpp/coqpp_parse.mly"
                                     ( GSymbParen _2 )
# 1468 "coqpp/coqpp_parse.ml"
               : 'doc_gram_gram_token))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'doc_gram_rules) in
    Obj.repr(
# 469 "coqpp/coqpp_parse.mly"
                                   ( GSymbProd _2 )
# 1475 "coqpp/coqpp_parse.ml"
               : 'doc_gram_gram_token))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 470 "coqpp/coqpp_parse.mly"
         ( GSymbString _1 )
# 1482 "coqpp/coqpp_parse.ml"
               : 'doc_gram_gram_token))
(* Entry file *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let file (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Coqpp_ast.t)
