let __coq_plugin_name = "ltac_plugin"
let _ = Mltop.add_known_module __coq_plugin_name

# 13 "plugins/ltac/g_ltac.mlg"
 

open Util
open Pp
open Glob_term
open Constrexpr
open Tacexpr
open Namegen
open Genarg
open Genredexpr
open Names
open Attributes

open Pcoq
open Pcoq.Prim
open Pcoq.Constr
open Pvernac.Vernac_
open Pltac

let fail_default_value = Locus.ArgArg 0

let arg_of_expr = function
    TacArg { CAst.v } -> v
  | e -> Tacexp (e:raw_tactic_expr)

let genarg_of_unit () = in_gen (rawwit Stdarg.wit_unit) ()
let genarg_of_int n = in_gen (rawwit Stdarg.wit_int) n
let genarg_of_ipattern pat = in_gen (rawwit Tacarg.wit_simple_intropattern) pat
let genarg_of_uconstr c = in_gen (rawwit Stdarg.wit_uconstr) c
let in_tac tac = in_gen (rawwit Tacarg.wit_ltac) tac

let reference_to_id qid =
  if Libnames.qualid_is_ident qid then
    CAst.make ?loc:qid.CAst.loc @@ Libnames.qualid_basename qid
  else
    CErrors.user_err ?loc:qid.CAst.loc
      (str "This expression should be a simple identifier.")

let tactic_mode = Entry.create "vernac:tactic_command"

let new_entry name =
  let e = Entry.create name in
  e

let toplevel_selector = new_entry "vernac:toplevel_selector"
let tacdef_body = new_entry "tactic:tacdef_body"

(* Registers [tactic_mode] as a parser for proof editing *)
let classic_proof_mode = Pvernac.register_proof_mode "Classic" tactic_mode

(* Hack to parse "[ id" without dropping [ *)
let test_bracket_ident =
  let open Pcoq.Lookahead in
  to_entry "test_bracket_ident" begin
    lk_kw "[" >> lk_ident
  end

(* Tactics grammar rules *)

let hint = G_proofs.hint



let _ = let tactic_then_last = Pcoq.Entry.make "tactic_then_last"
        and tactic_then_gen = Pcoq.Entry.make "tactic_then_gen"
        and tactic_then_locality = Pcoq.Entry.make "tactic_then_locality"
        and failkw = Pcoq.Entry.make "failkw"
        and tactic_arg_compat = Pcoq.Entry.make "tactic_arg_compat"
        and fresh_id = Pcoq.Entry.make "fresh_id"
        and tactic_atom = Pcoq.Entry.make "tactic_atom"
        and match_key = Pcoq.Entry.make "match_key"
        and input_fun = Pcoq.Entry.make "input_fun"
        and let_clause = Pcoq.Entry.make "let_clause"
        and match_pattern = Pcoq.Entry.make "match_pattern"
        and match_hyps = Pcoq.Entry.make "match_hyps"
        and match_context_rule = Pcoq.Entry.make "match_context_rule"
        and match_context_list = Pcoq.Entry.make "match_context_list"
        and match_rule = Pcoq.Entry.make "match_rule"
        and match_list = Pcoq.Entry.make "match_list"
        and message_token = Pcoq.Entry.make "message_token"
        and ltac_def_kind = Pcoq.Entry.make "ltac_def_kind"
        and range_selector = Pcoq.Entry.make "range_selector"
        and range_selector_or_nth = Pcoq.Entry.make "range_selector_or_nth"
        and selector_body = Pcoq.Entry.make "selector_body"
        and selector = Pcoq.Entry.make "selector"
        in
        let () =
        Pcoq.grammar_extend tactic_then_last
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 84 "plugins/ltac/g_ltac.mlg"
             [||] 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm tactic_expr))) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun lta _ loc -> 
# 83 "plugins/ltac/g_ltac.mlg"
          Array.map (function None -> TacId [] | Some t -> t) (Array.of_list lta) 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend tactic_then_gen
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 93 "plugins/ltac/g_ltac.mlg"
             ([TacId []], None) 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.nterm tactic_then_gen)))
                               (fun tg _ loc -> 
# 92 "plugins/ltac/g_ltac.mlg"
                                       let (first,last) = tg in (TacId [] :: first, last) 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun ta loc -> 
# 91 "plugins/ltac/g_ltac.mlg"
                              ([ta], None) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("..")))))
                                               ((Pcoq.Symbol.nterm tactic_then_last)))
                               (fun l _ loc -> 
# 90 "plugins/ltac/g_ltac.mlg"
                                        ([], Some (TacId [], l)) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tactic_expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("..")))))
                                               ((Pcoq.Symbol.nterm tactic_then_last)))
                               (fun l _ ta loc -> 
# 89 "plugins/ltac/g_ltac.mlg"
                                                          ([], Some (ta, l)) 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tactic_expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.nterm tactic_then_gen)))
                               (fun tg _ ta loc -> 
# 88 "plugins/ltac/g_ltac.mlg"
                                                         let (first,last) = tg in (ta::first, last) 
                                                   )])]}
        in let () =
        Pcoq.grammar_extend tactic_then_locality
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                ((Pcoq.Symbol.opt (Pcoq.Symbol.token (Tok.PKEYWORD (">"))))))
                                (fun l _ loc -> 
# 98 "plugins/ltac/g_ltac.mlg"
                            if Option.is_empty l then true else false 
                                                )])]}
        in let () =
        Pcoq.grammar_extend tactic_expr
        { Pcoq.pos=None; data=[(Some ("5"), Some (Gramlib.Gramext.RightA),
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm binder_tactic)))
                                (fun te loc -> 
# 102 "plugins/ltac/g_ltac.mlg"
                                te 
                                               )]);
                              (Some ("4"), Some (Gramlib.Gramext.LeftA),
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tactic_expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                                                               ((Pcoq.Symbol.nterm tactic_then_locality)))
                                                               ((Pcoq.Symbol.nterm tactic_then_gen)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ tg l _ ta0 loc -> 
# 106 "plugins/ltac/g_ltac.mlg"
                                                                                        
          let (first,tail) = tg in
          match l , tail with
          | false , Some (t,last) -> TacThen (ta0,TacExtendTac (Array.of_list first, t, last))
          | true  , Some (t,last) -> TacThens3parts (ta0, Array.of_list first, t, last)
          | false , None -> TacThen (ta0,TacDispatch first)
          | true  , None -> TacThens (ta0,first) 
                                                        );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta1 _ ta0 loc -> 
# 105 "plugins/ltac/g_ltac.mlg"
                                                       TacThen (ta0,ta1) 
                                                    );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                                              ((Pcoq.Symbol.nterm binder_tactic)))
                              (fun ta1 _ ta0 loc -> 
# 104 "plugins/ltac/g_ltac.mlg"
                                                         TacThen (ta0, ta1) 
                                                    )]);
                              (Some ("3"), Some (Gramlib.Gramext.RightA),
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm selector)))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun ta sel loc -> 
# 127 "plugins/ltac/g_ltac.mlg"
                                              TacSelect (sel, ta) 
                                                  );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("abstract"))))))
                                                              (Pcoq.Symbol.next))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("using")))))
                                              ((Pcoq.Symbol.nterm ident)))
                              (fun s _ tc _ loc -> 
# 126 "plugins/ltac/g_ltac.mlg"
          TacAbstract (tc,Some s) 
                                                   );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("abstract"))))))
                                              (Pcoq.Symbol.next))
                              (fun tc _ loc -> 
# 124 "plugins/ltac/g_ltac.mlg"
                                         TacAbstract (tc,None) 
                                               );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("infoH"))))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta _ loc -> 
# 122 "plugins/ltac/g_ltac.mlg"
                                             TacShowHyps ta 
                                               );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("exactly_once"))))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta _ loc -> 
# 121 "plugins/ltac/g_ltac.mlg"
                                                    TacExactlyOnce ta 
                                               );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("once"))))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta _ loc -> 
# 120 "plugins/ltac/g_ltac.mlg"
                                            TacOnce ta 
                                               );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("progress"))))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta _ loc -> 
# 119 "plugins/ltac/g_ltac.mlg"
                                                TacProgress ta 
                                               );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("repeat"))))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta _ loc -> 
# 118 "plugins/ltac/g_ltac.mlg"
                                              TacRepeat ta 
                                               );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("time"))))))
                                                              ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm string))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta s _ loc -> 
# 117 "plugins/ltac/g_ltac.mlg"
                                                            TacTime (s,ta) 
                                                 );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("timeout"))))))
                                                              ((Pcoq.Symbol.nterm int_or_var)))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta n _ loc -> 
# 116 "plugins/ltac/g_ltac.mlg"
                                                               TacTimeout (n,ta) 
                                                 );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("do"))))))
                                                              ((Pcoq.Symbol.nterm int_or_var)))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta n _ loc -> 
# 115 "plugins/ltac/g_ltac.mlg"
                                                          TacDo (n,ta) 
                                                 );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("try"))))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta _ loc -> 
# 114 "plugins/ltac/g_ltac.mlg"
                                           TacTry ta 
                                               )]);
                              (Some ("2"), Some (Gramlib.Gramext.RightA),
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tactic_expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("||")))))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun ta1 _ ta0 loc -> 
# 136 "plugins/ltac/g_ltac.mlg"
                                                        TacOrelse (ta0,ta1) 
                                                     );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("||")))))
                                              ((Pcoq.Symbol.nterm binder_tactic)))
                              (fun ta1 _ ta0 loc -> 
# 135 "plugins/ltac/g_ltac.mlg"
                                                          TacOrelse (ta0,ta1) 
                                                    );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("tryif"))))))
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("then")))))
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("else")))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun tae _ tat _ ta _ loc -> 
# 134 "plugins/ltac/g_ltac.mlg"
                                              TacIfThenCatch(ta,tat,tae) 
                                                           );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("+")))))
                                              ((Pcoq.Symbol.nterm tactic_expr)))
                              (fun ta1 _ ta0 loc -> 
# 131 "plugins/ltac/g_ltac.mlg"
                                                       TacOr (ta0,ta1) 
                                                    );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("+")))))
                                              ((Pcoq.Symbol.nterm binder_tactic)))
                              (fun ta1 _ ta0 loc -> 
# 130 "plugins/ltac/g_ltac.mlg"
                                                         TacOr (ta0,ta1) 
                                                    )]);
                              (Some ("1"), Some (Gramlib.Gramext.RightA),
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm reference)))
                                               ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm tactic_arg_compat))))
                               (fun la r loc -> 
# 155 "plugins/ltac/g_ltac.mlg"
          TacArg(CAst.make ~loc @@ TacCall (CAst.make ~loc (r,la))) 
                                                );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.nterm tactic_arg)))
                              (fun a loc -> 
# 153 "plugins/ltac/g_ltac.mlg"
                            TacArg(CAst.make ~loc a) 
                                            );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.nterm simple_tactic)))
                              (fun st loc -> 
# 152 "plugins/ltac/g_ltac.mlg"
                                st 
                                             );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm failkw)))
                                                              ((Pcoq.Symbol.rules 
                                                              [Pcoq.Rules.make 
                                                              (Pcoq.Rule.stop)
                                                              (fun loc -> 
                                                              
# 150 "plugins/ltac/g_ltac.mlg"
                                                       fail_default_value 
                                                              );
                                                              Pcoq.Rules.make 
                                                              (Pcoq.Rule.next_norec 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm int_or_var)))
                                                              (fun n loc -> 
                                                              
# 150 "plugins/ltac/g_ltac.mlg"
                                            n 
                                                              )])))
                                              ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm message_token))))
                              (fun l n g loc -> 
# 151 "plugins/ltac/g_ltac.mlg"
                                       TacFail (g,n,l) 
                                                );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("idtac"))))))
                                              ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm message_token))))
                              (fun l _ loc -> 
# 149 "plugins/ltac/g_ltac.mlg"
                                                    TacId l 
                                              );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("solve"))))))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                              ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm tactic_expr)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                              (fun _ l _ _ loc -> 
# 148 "plugins/ltac/g_ltac.mlg"
            TacSolve l 
                                                  );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("first"))))))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                              ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm tactic_expr)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                              (fun _ l _ _ loc -> 
# 146 "plugins/ltac/g_ltac.mlg"
            TacFirst l 
                                                  );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm match_key)))
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                              ((Pcoq.Symbol.nterm match_list)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("end")))))
                              (fun _ mrl _ c b loc -> 
# 144 "plugins/ltac/g_ltac.mlg"
            TacMatch (b,c,mrl) 
                                                      );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm match_key)))
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("reverse"))))))
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("goal"))))))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                              ((Pcoq.Symbol.nterm match_context_list)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("end")))))
                              (fun _ mrl _ _ _ b loc -> 
# 142 "plugins/ltac/g_ltac.mlg"
            TacMatchGoal (b,true,mrl) 
                                                        );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm match_key)))
                                                              ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                              ("goal"))))))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                              ((Pcoq.Symbol.nterm match_context_list)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("end")))))
                              (fun _ mrl _ _ b loc -> 
# 139 "plugins/ltac/g_ltac.mlg"
            TacMatchGoal (b,false,mrl) 
                                                      )]);
                              (Some ("0"), None,
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tactic_atom)))
                               (fun a loc -> 
# 164 "plugins/ltac/g_ltac.mlg"
                             TacArg (CAst.make ~loc a) 
                                             );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (">")))))
                                                              ((Pcoq.Symbol.nterm tactic_then_gen)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                              (fun _ tg _ _ loc -> 
# 158 "plugins/ltac/g_ltac.mlg"
                                                
          let (tf,tail) = tg in
          begin match tail with
          | Some (t,tl) -> TacExtendTac(Array.of_list tf,t,tl)
          | None -> TacDispatch tf
          end 
                                                   );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                              ((Pcoq.Symbol.nterm tactic_expr)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                              (fun _ a _ loc -> 
# 157 "plugins/ltac/g_ltac.mlg"
                                       a 
                                                )])]}
        in let () =
        Pcoq.grammar_extend failkw
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                ("gfail"))))))
                                (fun _ loc -> 
# 167 "plugins/ltac/g_ltac.mlg"
                                                        TacGlobal 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("fail"))))))
                               (fun _ loc -> 
# 167 "plugins/ltac/g_ltac.mlg"
                        TacLocal 
                                             )])]}
        in let () =
        Pcoq.grammar_extend binder_tactic
        { Pcoq.pos=None; data=[(None, Some (Gramlib.Gramext.RightA),
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("info"))))))
                                                ((Pcoq.Symbol.nterml tactic_expr ("5"))))
                                (fun tc _ loc -> 
# 177 "plugins/ltac/g_ltac.mlg"
                                                      TacInfo tc 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("let")))))
                                                               ((Pcoq.Symbol.rules 
                                                               [Pcoq.Rules.make 
                                                               (Pcoq.Rule.stop)
                                                               (fun loc -> 
                                                               
# 174 "plugins/ltac/g_ltac.mlg"
                                                       false 
                                                               );
                                                               Pcoq.Rules.make 
                                                               (Pcoq.Rule.next_norec 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("rec"))))))
                                                               (fun _ loc ->
                                                               
# 174 "plugins/ltac/g_ltac.mlg"
                                         true 
                                                               )])))
                                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm let_clause)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))) false)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("in")))))
                                               ((Pcoq.Symbol.nterml tactic_expr ("5"))))
                               (fun body _ llc isrec _ loc -> 
# 176 "plugins/ltac/g_ltac.mlg"
                                            TacLetIn (isrec,llc,body) 
                                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("fun")))))
                                                               ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm input_fun)))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                               ((Pcoq.Symbol.nterml tactic_expr ("5"))))
                               (fun body _ it _ loc -> 
# 173 "plugins/ltac/g_ltac.mlg"
            TacFun (it,body) 
                                                       )])]}
        in let () =
        Pcoq.grammar_extend tactic_arg_compat
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("()")))))
                                (fun _ loc -> 
# 184 "plugins/ltac/g_ltac.mlg"
                  TacGeneric (genarg_of_unit ()) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Constr.constr)))
                               (fun c loc -> 
# 182 "plugins/ltac/g_ltac.mlg"
                               (match c with { CAst.v = CRef (r,None) } -> Reference r | c -> ConstrMayEval (ConstrTerm c)) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tactic_arg)))
                               (fun a loc -> 
# 181 "plugins/ltac/g_ltac.mlg"
                            a 
                                             )])]}
        in let () =
        Pcoq.grammar_extend tactic_arg
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                ("numgoals"))))))
                                (fun _ loc -> 
# 191 "plugins/ltac/g_ltac.mlg"
                              TacNumgoals 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("type_term"))))))
                                               ((Pcoq.Symbol.nterm uconstr)))
                               (fun c _ loc -> 
# 190 "plugins/ltac/g_ltac.mlg"
                                          TacPretype c 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("fresh"))))))
                                               ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm fresh_id))))
                               (fun l _ loc -> 
# 189 "plugins/ltac/g_ltac.mlg"
                                               TacFreshId l 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm constr_eval)))
                               (fun c loc -> 
# 188 "plugins/ltac/g_ltac.mlg"
                             ConstrMayEval c 
                                             )])]}
        in let () =
        Pcoq.grammar_extend fresh_id
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm qualid)))
                                (fun qid loc -> 
# 198 "plugins/ltac/g_ltac.mlg"
                            Locus.ArgVar (CAst.make ~loc @@ Libnames.qualid_basename qid) 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PSTRING (None)))))
                               (fun s loc -> 
# 197 "plugins/ltac/g_ltac.mlg"
                        Locus.ArgArg s (*| id = ident -> Locus.ArgVar (!@loc,id)*) 
                                             )])]}
        in let () =
        Pcoq.grammar_extend constr_eval
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("type"))))))
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("of"))))))
                                                ((Pcoq.Symbol.nterm Constr.constr)))
                                (fun c _ _ loc -> 
# 206 "plugins/ltac/g_ltac.mlg"
            ConstrTypeOf c 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("context"))))))
                                                               ((Pcoq.Symbol.nterm identref)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ c _ id _ loc -> 
# 204 "plugins/ltac/g_ltac.mlg"
            ConstrContext (id,c) 
                                                      );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("eval"))))))
                                                               ((Pcoq.Symbol.nterm red_expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("in")))))
                                               ((Pcoq.Symbol.nterm Constr.constr)))
                               (fun c _ rtc _ loc -> 
# 202 "plugins/ltac/g_ltac.mlg"
            ConstrEval (rtc,c) 
                                                     )])]}
        in let () =
        Pcoq.grammar_extend constr_may_eval
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Constr.constr)))
                                (fun c loc -> 
# 210 "plugins/ltac/g_ltac.mlg"
                               ConstrTerm c 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm constr_eval)))
                               (fun c loc -> 
# 209 "plugins/ltac/g_ltac.mlg"
                             c 
                                             )])]}
        in let () =
        Pcoq.grammar_extend tactic_atom
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("()")))))
                                (fun _ loc -> 
# 215 "plugins/ltac/g_ltac.mlg"
                  TacGeneric (genarg_of_unit ()) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm reference)))
                               (fun r loc -> 
# 214 "plugins/ltac/g_ltac.mlg"
                           TacCall (CAst.make ~loc (r,[])) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm integer)))
                               (fun n loc -> 
# 213 "plugins/ltac/g_ltac.mlg"
                         TacGeneric (genarg_of_int n) 
                                             )])]}
        in let () =
        Pcoq.grammar_extend match_key
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                ("multimatch"))))))
                                (fun _ loc -> 
# 220 "plugins/ltac/g_ltac.mlg"
                                General 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("lazymatch"))))))
                               (fun _ loc -> 
# 219 "plugins/ltac/g_ltac.mlg"
                               Select 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("match")))))
                               (fun _ loc -> 
# 218 "plugins/ltac/g_ltac.mlg"
                     Once 
                                             )])]}
        in let () =
        Pcoq.grammar_extend input_fun
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm ident)))
                                (fun l loc -> 
# 224 "plugins/ltac/g_ltac.mlg"
                       Name.Name l 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                               (fun _ loc -> 
# 223 "plugins/ltac/g_ltac.mlg"
                 Name.Anonymous 
                                             )])]}
        in let () =
        Pcoq.grammar_extend let_clause
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm identref)))
                                                                ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm input_fun)))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                ((Pcoq.Symbol.nterm tactic_expr)))
                                (fun te _ args idr loc -> 
# 232 "plugins/ltac/g_ltac.mlg"
           (CAst.map (fun id -> Name id) idr, arg_of_expr (TacFun(args,te))) 
                                                          );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.rules 
                                                               [Pcoq.Rules.make 
                                                               (Pcoq.Rule.next_norec 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                                                               (fun _ loc ->
                                                               
# 229 "plugins/ltac/g_ltac.mlg"
                       CAst.make ~loc Anonymous 
                                                               )])))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun te _ na loc -> 
# 230 "plugins/ltac/g_ltac.mlg"
           (na, arg_of_expr te) 
                                                   );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm identref)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun te _ idr loc -> 
# 228 "plugins/ltac/g_ltac.mlg"
           (CAst.map (fun id -> Name id) idr, arg_of_expr te) 
                                                    )])]}
        in let () =
        Pcoq.grammar_extend match_pattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Constr.lconstr_pattern)))
                                (fun pc loc -> 
# 238 "plugins/ltac/g_ltac.mlg"
                                         Term pc 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("context"))))))
                                                               ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm Constr.ident))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr_pattern)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ pc _ oid _ loc -> 
# 237 "plugins/ltac/g_ltac.mlg"
          Subterm (oid, pc) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend match_hyps
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm name)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                ((Pcoq.Symbol.nterm match_pattern)))
                                (fun mpv _ na loc -> 
# 244 "plugins/ltac/g_ltac.mlg"
          let t, ty =
            match mpv with
            | Term t -> (match t with
              | { CAst.v = CCast (t, (CastConv ty | CastVM ty | CastNative ty)) } -> Term t, Some (Term ty)
              | _ -> mpv, None)
            | _ -> mpv, None
          in Def (na, t, Option.default (Term (CAst.make @@ CHole (None, IntroAnonymous, None))) ty) 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm name)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.nterm match_pattern)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                               ((Pcoq.Symbol.nterm match_pattern)))
                               (fun mpt _ _ mpv _ _ na loc -> 
# 242 "plugins/ltac/g_ltac.mlg"
                                                                                      Def (na, mpv, mpt) 
                                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm name)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                               ((Pcoq.Symbol.nterm match_pattern)))
                               (fun mp _ na loc -> 
# 241 "plugins/ltac/g_ltac.mlg"
                                                 Hyp (na, mp) 
                                                   )])]}
        in let () =
        Pcoq.grammar_extend match_context_rule
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                                ((Pcoq.Symbol.nterm tactic_expr)))
                                (fun te _ _ loc -> 
# 258 "plugins/ltac/g_ltac.mlg"
                                         All te 
                                                   );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm match_hyps)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                                               ((Pcoq.Symbol.nterm match_pattern)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun te _ _ mp _ largs _ loc -> 
# 257 "plugins/ltac/g_ltac.mlg"
                                         Pat (largs, mp, te) 
                                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm match_hyps)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                                               ((Pcoq.Symbol.nterm match_pattern)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun te _ mp _ largs loc -> 
# 255 "plugins/ltac/g_ltac.mlg"
                                    Pat (largs, mp, te) 
                                                           )])]}
        in let () =
        Pcoq.grammar_extend match_context_list
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm match_context_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                (fun mrl _ loc -> 
# 262 "plugins/ltac/g_ltac.mlg"
                                                         mrl 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm match_context_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun mrl loc -> 
# 261 "plugins/ltac/g_ltac.mlg"
                                                    mrl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend match_rule
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                                ((Pcoq.Symbol.nterm tactic_expr)))
                                (fun te _ _ loc -> 
# 266 "plugins/ltac/g_ltac.mlg"
                                         All te 
                                                   );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm match_pattern)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun te _ mp loc -> 
# 265 "plugins/ltac/g_ltac.mlg"
                                                        Pat ([],mp,te) 
                                                   )])]}
        in let () =
        Pcoq.grammar_extend match_list
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm match_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                (fun mrl _ loc -> 
# 270 "plugins/ltac/g_ltac.mlg"
                                                 mrl 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm match_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun mrl loc -> 
# 269 "plugins/ltac/g_ltac.mlg"
                                            mrl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend message_token
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm integer)))
                                (fun n loc -> 
# 275 "plugins/ltac/g_ltac.mlg"
                         MsgInt n 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PSTRING (None)))))
                               (fun s loc -> 
# 274 "plugins/ltac/g_ltac.mlg"
                        MsgString s 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm identref)))
                               (fun id loc -> 
# 273 "plugins/ltac/g_ltac.mlg"
                           MsgIdent id 
                                              )])]}
        in let () =
        Pcoq.grammar_extend ltac_def_kind
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("::=")))))
                                (fun _ loc -> 
# 280 "plugins/ltac/g_ltac.mlg"
                   true 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                               (fun _ loc -> 
# 279 "plugins/ltac/g_ltac.mlg"
                  false 
                                             )])]}
        in let () =
        Pcoq.grammar_extend tacdef_body
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Constr.global)))
                                                                ((Pcoq.Symbol.nterm ltac_def_kind)))
                                                ((Pcoq.Symbol.nterm tactic_expr)))
                                (fun body redef name loc -> 
# 293 "plugins/ltac/g_ltac.mlg"
          if redef then Tacexpr.TacticRedefinition (name, body)
          else
            let id = reference_to_id name in
            Tacexpr.TacticDefinition (id, body) 
                                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm Constr.global)))
                                                               ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm input_fun)))))
                                                               ((Pcoq.Symbol.nterm ltac_def_kind)))
                                               ((Pcoq.Symbol.nterm tactic_expr)))
                               (fun body redef it name loc -> 
# 287 "plugins/ltac/g_ltac.mlg"
          if redef then Tacexpr.TacticRedefinition (name, TacFun (it, body))
          else
            let id = reference_to_id name in
            Tacexpr.TacticDefinition (id, TacFun (it, body)) 
                                                              )])]}
        in let () =
        Pcoq.grammar_extend tactic
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm tactic_expr)))
                                (fun tac loc -> 
# 300 "plugins/ltac/g_ltac.mlg"
                               tac 
                                                )])]}
        in let () =
        Pcoq.grammar_extend range_selector
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm natural)))
                                (fun n loc -> 
# 305 "plugins/ltac/g_ltac.mlg"
                         (n, n) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm natural)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("-")))))
                                               ((Pcoq.Symbol.nterm natural)))
                               (fun m _ n loc -> 
# 304 "plugins/ltac/g_ltac.mlg"
                                             (n, m) 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend range_selector_or_nth
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm natural)))
                                                ((Pcoq.Symbol.opt (Pcoq.Symbol.rules 
                                                [Pcoq.Rules.make (Pcoq.Rule.next_norec 
                                                                 (Pcoq.Rule.next_norec 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))))
                                                                 ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm range_selector)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                                 (fun l _
                                                                 loc -> 
                                                                 
# 314 "plugins/ltac/g_ltac.mlg"
                                                            l 
                                                                 )]))))
                                (fun l n loc -> 
# 315 "plugins/ltac/g_ltac.mlg"
          let open Goal_select in
          Option.cata (fun l -> SelectList ((n, n) :: l)) (SelectNth n) l 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm natural)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("-")))))
                                                               ((Pcoq.Symbol.nterm natural)))
                                               ((Pcoq.Symbol.opt (Pcoq.Symbol.rules 
                                               [Pcoq.Rules.make (Pcoq.Rule.next_norec 
                                                                (Pcoq.Rule.next_norec 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))))
                                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm range_selector)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                                (fun l _
                                                                loc -> 
                                                                
# 311 "plugins/ltac/g_ltac.mlg"
                                                            l 
                                                                )]))))
                               (fun l m _ n loc -> 
# 312 "plugins/ltac/g_ltac.mlg"
          Goal_select.SelectList ((n, m) :: Option.default [] l) 
                                                   )])]}
        in let () =
        Pcoq.grammar_extend selector_body
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm test_bracket_ident)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                                ((Pcoq.Symbol.nterm ident)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                                (fun _ id _ _ loc -> 
# 320 "plugins/ltac/g_ltac.mlg"
                                                    Goal_select.SelectId id 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm range_selector_or_nth)))
                               (fun l loc -> 
# 319 "plugins/ltac/g_ltac.mlg"
                                     l 
                                             )])]}
        in let () =
        Pcoq.grammar_extend selector
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("only"))))))
                                                                ((Pcoq.Symbol.nterm selector_body)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                (fun _ sel _ loc -> 
# 323 "plugins/ltac/g_ltac.mlg"
                                                    sel 
                                                    )])]}
        in let () =
        Pcoq.grammar_extend toplevel_selector
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("all"))))))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                (fun _ _ loc -> 
# 328 "plugins/ltac/g_ltac.mlg"
                              Goal_select.SelectAll 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("!")))))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                               (fun _ _ loc -> 
# 327 "plugins/ltac/g_ltac.mlg"
                      Goal_select.SelectAlreadyFocused 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm selector_body)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                               (fun _ sel loc -> 
# 326 "plugins/ltac/g_ltac.mlg"
                                      sel 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend tactic_mode
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm toplevel_selector))))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("{")))))
                                (fun _ g loc -> 
# 332 "plugins/ltac/g_ltac.mlg"
                                            Vernacexpr.VernacSubproof g 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm toplevel_selector))))
                                               ((Pcoq.Symbol.nterm G_vernac.query_command)))
                               (fun tac g loc -> 
# 331 "plugins/ltac/g_ltac.mlg"
                                                                     tac g 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend command
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("Proof"))))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("using")))))
                                                                ((Pcoq.Symbol.nterm G_vernac.section_subset_expr)))
                                                ((Pcoq.Symbol.opt (Pcoq.Symbol.rules 
                                                [Pcoq.Rules.make (Pcoq.Rule.next_norec 
                                                                 (Pcoq.Rule.next_norec 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                                 ((Pcoq.Symbol.nterm Pltac.tactic)))
                                                                 (fun ta _
                                                                 loc -> 
                                                                 
# 339 "plugins/ltac/g_ltac.mlg"
                                                  in_tac ta 
                                                                 )]))))
                                (fun ta l _ _ loc -> 
# 340 "plugins/ltac/g_ltac.mlg"
            Vernacexpr.VernacProof (ta,Some l) 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("Proof"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                               ((Pcoq.Symbol.nterm Pltac.tactic)))
                                               ((Pcoq.Symbol.opt (Pcoq.Symbol.rules 
                                               [Pcoq.Rules.make (Pcoq.Rule.next_norec 
                                                                (Pcoq.Rule.next_norec 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("using")))))
                                                                ((Pcoq.Symbol.nterm G_vernac.section_subset_expr)))
                                                                (fun l _
                                                                loc -> 
                                                                
# 336 "plugins/ltac/g_ltac.mlg"
                                                                 l 
                                                                )]))))
                               (fun l ta _ _ loc -> 
# 337 "plugins/ltac/g_ltac.mlg"
            Vernacexpr.VernacProof (Some (in_tac ta), l) 
                                                    )])]}
        in let () =
        Pcoq.grammar_extend hint
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("Extern"))))))
                                                                ((Pcoq.Symbol.nterm natural)))
                                                                ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm Constr.constr_pattern))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                                ((Pcoq.Symbol.nterm Pltac.tactic)))
                                (fun tac _ c n _ loc -> 
# 345 "plugins/ltac/g_ltac.mlg"
          Vernacexpr.HintsExtern (n,c, in_tac tac) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend operconstr
        { Pcoq.pos=Some
        (Gramlib.Gramext.Level "0"); data=[(None, None,
                                           [Pcoq.Production.make
                                            (Pcoq.Rule.next (Pcoq.Rule.next 
                                                            (Pcoq.Rule.next 
                                                            (Pcoq.Rule.next 
                                                            (Pcoq.Rule.next 
                                                            (Pcoq.Rule.stop)
                                                            ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                            ("ltac"))))))
                                                            ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                            ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                            ((Pcoq.Symbol.nterm Pltac.tactic_expr)))
                                                            ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                            (fun _ tac _ _ _ loc -> 
# 349 "plugins/ltac/g_ltac.mlg"
          let arg = Genarg.in_gen (Genarg.rawwit Tacarg.wit_tactic) tac in
          CAst.make ~loc @@ CHole (None, IntroAnonymous, Some arg) 
                                                                    )])]}
        in ()


# 354 "plugins/ltac/g_ltac.mlg"
 

open Stdarg
open Tacarg
open Vernacextend
open Goptions
open Libnames

let print_info_trace =
  declare_intopt_option_and_ref ~depr:false ~key:["Info" ; "Level"]

let vernac_solve ~pstate n info tcom b =
  let open Goal_select in
  let pstate, status = Declare.Proof.map_fold_proof_endline (fun etac p ->
      let with_end_tac = if b then Some etac else None in
      let global = match n with SelectAll | SelectList _ -> true | _ -> false in
      let info = Option.append info (print_info_trace ()) in
      let (p,status) =
        Proof.solve n info (Tacinterp.hide_interp global tcom None) ?with_end_tac p
      in
      (* in case a strict subtree was completed,
         go back to the top of the prooftree *)
      let p = Proof.maximal_unfocus Vernacentries.command_focus p in
      p,status) pstate in
  if not status then Feedback.feedback Feedback.AddedAxiom;
  pstate

let pr_ltac_selector s = Pptactic.pr_goal_selector ~toplevel:true s



let (wit_ltac_selector, ltac_selector) = Vernacextend.vernac_argument_extend ~name:"ltac_selector" 
                                         {
                                         Vernacextend.arg_parsing = Vernacextend.Arg_alias (toplevel_selector);
                                         Vernacextend.arg_printer = fun env sigma -> 
                                         
# 385 "plugins/ltac/g_ltac.mlg"
                                                  pr_ltac_selector 
                                         ;
                                         }
let _ = (wit_ltac_selector, ltac_selector)


# 389 "plugins/ltac/g_ltac.mlg"
 

let pr_ltac_info n = str "Info" ++ spc () ++ int n



let (wit_ltac_info, ltac_info) = Vernacextend.vernac_argument_extend ~name:"ltac_info" 
                                 {
                                 Vernacextend.arg_parsing = Vernacextend.Arg_rules (
                                                            [(Pcoq.Production.make
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (CLexer.terminal "Info"))))
                                                              ((Pcoq.Symbol.nterm natural)))
                                                              (fun n _ loc ->
                                                              
# 396 "plugins/ltac/g_ltac.mlg"
                             n 
                                                              ))]);
                                 Vernacextend.arg_printer = fun env sigma -> 
                                 
# 395 "plugins/ltac/g_ltac.mlg"
                                              pr_ltac_info 
                                 ;
                                 }
let _ = (wit_ltac_info, ltac_info)


# 399 "plugins/ltac/g_ltac.mlg"
 

let pr_ltac_use_default b =
  if b then (* Bug: a space is inserted before "..." *) str ".." else mt ()



let (wit_ltac_use_default, ltac_use_default) = Vernacextend.vernac_argument_extend ~name:"ltac_use_default" 
                                               {
                                               Vernacextend.arg_parsing = 
                                               Vernacextend.Arg_rules (
                                               [(Pcoq.Production.make
                                                 (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.token (CLexer.terminal "..."))))
                                                 (fun _ loc -> 
# 408 "plugins/ltac/g_ltac.mlg"
                 true 
                                                               ));
                                               (Pcoq.Production.make
                                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (CLexer.terminal "."))))
                                                (fun _ loc -> 
# 407 "plugins/ltac/g_ltac.mlg"
               false 
                                                              ))]);
                                               Vernacextend.arg_printer = fun env sigma -> 
                                               
# 406 "plugins/ltac/g_ltac.mlg"
                                                     pr_ltac_use_default 
                                               ;
                                               }
let _ = (wit_ltac_use_default, ltac_use_default)


# 411 "plugins/ltac/g_ltac.mlg"
 

let is_anonymous_abstract = function
  | TacAbstract (_,None) -> true
  | TacSolve [TacAbstract (_,None)] -> true
  | _ -> false
let rm_abstract = function
  | TacAbstract (t,_) -> t
  | TacSolve [TacAbstract (t,_)] -> TacSolve [t]
  | x -> x
let is_explicit_terminator = function TacSolve _ -> true | _ -> false



let () = Vernacextend.vernac_extend ~command:"VernacSolve"  ?entry:(Some ( tactic_mode )) 
         [(Vernacextend.TyML (false, Vernacextend.TyNonTerminal (Extend.TUopt (
                                                                 Extend.TUentry (Genarg.get_arg_tag wit_ltac_selector)), 
                                     Vernacextend.TyNonTerminal (Extend.TUopt (
                                                                 Extend.TUentry (Genarg.get_arg_tag wit_ltac_info)), 
                                     Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_tactic), 
                                     Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_ltac_use_default), 
                                     Vernacextend.TyNil)))), (let coqpp_body g
                                                             n t def
                                                             () = Vernacextend.VtModifyProof (
                                                                  
# 427 "plugins/ltac/g_ltac.mlg"
                                  
    let g = Option.default (Goal_select.get_default_goal_selector ()) g in
    vernac_solve g n t def
  
                                                                  ) in fun g
                                                             n t def ~atts
                                                             -> coqpp_body g
                                                             n t def
                                                             (Attributes.unsupported_attributes atts)), Some 
         (fun g n t def -> 
# 427 "plugins/ltac/g_ltac.mlg"
      classify_as_proofstep 
         )));
         (Vernacextend.TyML (false, Vernacextend.TyTerminal ("par", Vernacextend.TyTerminal (":", 
                                                                    Vernacextend.TyNonTerminal (
                                                                    Extend.TUopt (
                                                                    Extend.TUentry (Genarg.get_arg_tag wit_ltac_info)), 
                                                                    Vernacextend.TyNonTerminal (
                                                                    Extend.TUentry (Genarg.get_arg_tag wit_tactic), 
                                                                    Vernacextend.TyNonTerminal (
                                                                    Extend.TUentry (Genarg.get_arg_tag wit_ltac_use_default), 
                                                                    Vernacextend.TyNil))))), 
         (let coqpp_body n t def () = Vernacextend.VtModifyProof (
# 438 "plugins/ltac/g_ltac.mlg"
          
      let t = rm_abstract t in
      vernac_solve Goal_select.SelectAll n t def
    
                                      ) in fun n
         t def ~atts -> coqpp_body n t def
         (Attributes.unsupported_attributes atts)), Some (fun n t def -> 
# 432 "plugins/ltac/g_ltac.mlg"
     
      let anon_abstracting_tac = is_anonymous_abstract t in
      let solving_tac = is_explicit_terminator t in
      let parallel = `Yes (solving_tac,anon_abstracting_tac) in
      let pbr = if solving_tac then Some "par" else None in
      VtProofStep{ parallel = parallel; proof_block_detection = pbr }
    
                                                         )))]


# 444 "plugins/ltac/g_ltac.mlg"
 

let pr_ltac_tactic_level n = str "(at level " ++ int n ++ str ")"



let (wit_ltac_tactic_level, ltac_tactic_level) = Vernacextend.vernac_argument_extend ~name:"ltac_tactic_level" 
                                                 {
                                                 Vernacextend.arg_parsing = 
                                                 Vernacextend.Arg_rules (
                                                 [(Pcoq.Production.make
                                                   (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                   (Pcoq.Rule.next 
                                                                   (Pcoq.Rule.next 
                                                                   (Pcoq.Rule.next 
                                                                   (Pcoq.Rule.stop)
                                                                   ((Pcoq.Symbol.token (CLexer.terminal "("))))
                                                                   ((Pcoq.Symbol.token (CLexer.terminal "at"))))
                                                                   ((Pcoq.Symbol.token (CLexer.terminal "level"))))
                                                                   ((Pcoq.Symbol.nterm natural)))
                                                                   ((Pcoq.Symbol.token (CLexer.terminal ")"))))
                                                   (fun _ n _ _ _ loc -> 
# 451 "plugins/ltac/g_ltac.mlg"
                                           n 
                                                                    ))]);
                                                 Vernacextend.arg_printer = fun env sigma -> 
                                                 
# 450 "plugins/ltac/g_ltac.mlg"
                                                      pr_ltac_tactic_level 
                                                 ;
                                                 }
let _ = (wit_ltac_tactic_level, ltac_tactic_level)

let (wit_ltac_production_sep, ltac_production_sep) = Vernacextend.vernac_argument_extend ~name:"ltac_production_sep" 
                                                     {
                                                     Vernacextend.arg_parsing = 
                                                     Vernacextend.Arg_rules (
                                                     [(Pcoq.Production.make
                                                       (Pcoq.Rule.next 
                                                       (Pcoq.Rule.next 
                                                       (Pcoq.Rule.stop)
                                                       ((Pcoq.Symbol.token (CLexer.terminal ","))))
                                                       ((Pcoq.Symbol.nterm string)))
                                                       (fun sep _ loc -> 
# 455 "plugins/ltac/g_ltac.mlg"
                           sep 
                                                                    ))]);
                                                     Vernacextend.arg_printer = fun env sigma -> 
                                                     fun _ -> Pp.str "missing printer";
                                                     }
let _ = (wit_ltac_production_sep, ltac_production_sep)


# 458 "plugins/ltac/g_ltac.mlg"
 

let pr_ltac_production_item = function
| Tacentries.TacTerm s -> quote (str s)
| Tacentries.TacNonTerm (_, ((arg, None), None)) -> str arg
| Tacentries.TacNonTerm (_, ((arg, Some _), None)) -> assert false
| Tacentries.TacNonTerm (_, ((arg, sep), Some id)) ->
  let sep = match sep with
  | None -> mt ()
  | Some sep -> str "," ++ spc () ++ quote (str sep)
  in
  str arg ++ str "(" ++ Id.print id ++ sep ++ str ")"



let (wit_ltac_production_item, ltac_production_item) = Vernacextend.vernac_argument_extend ~name:"ltac_production_item" 
                                                       {
                                                       Vernacextend.arg_parsing = 
                                                       Vernacextend.Arg_rules (
                                                       [(Pcoq.Production.make
                                                         (Pcoq.Rule.next 
                                                         (Pcoq.Rule.stop)
                                                         ((Pcoq.Symbol.nterm ident)))
                                                         (fun nt loc -> 
# 478 "plugins/ltac/g_ltac.mlg"
    Tacentries.TacNonTerm (Loc.tag ~loc ((Id.to_string nt, None), None)) 
                                                                    ));
                                                       (Pcoq.Production.make
                                                        (Pcoq.Rule.next 
                                                        (Pcoq.Rule.next 
                                                        (Pcoq.Rule.next 
                                                        (Pcoq.Rule.next 
                                                        (Pcoq.Rule.next 
                                                        (Pcoq.Rule.stop)
                                                        ((Pcoq.Symbol.nterm ident)))
                                                        ((Pcoq.Symbol.token (CLexer.terminal "("))))
                                                        ((Pcoq.Symbol.nterm ident)))
                                                        ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm ltac_production_sep))))
                                                        ((Pcoq.Symbol.token (CLexer.terminal ")"))))
                                                        (fun _ sep p _ nt
                                                        loc -> 
# 476 "plugins/ltac/g_ltac.mlg"
    Tacentries.TacNonTerm (Loc.tag ~loc ((Id.to_string nt, sep), Some p)) 
                                                               ));
                                                       (Pcoq.Production.make
                                                        (Pcoq.Rule.next 
                                                        (Pcoq.Rule.stop)
                                                        ((Pcoq.Symbol.nterm string)))
                                                        (fun s loc -> 
# 474 "plugins/ltac/g_ltac.mlg"
                     Tacentries.TacTerm s 
                                                                    ))]);
                                                       Vernacextend.arg_printer = fun env sigma -> 
                                                       
# 473 "plugins/ltac/g_ltac.mlg"
                                                         pr_ltac_production_item 
                                                       ;
                                                       }
let _ = (wit_ltac_production_item, ltac_production_item)

let () = Vernacextend.vernac_extend ~command:"VernacTacticNotation"  ?entry:None 
         [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Tactic", 
                                     Vernacextend.TyTerminal ("Notation", 
                                     Vernacextend.TyNonTerminal (Extend.TUopt (
                                                                 Extend.TUentry (Genarg.get_arg_tag wit_ltac_tactic_level)), 
                                     Vernacextend.TyNonTerminal (Extend.TUlist1 (
                                                                 Extend.TUentry (Genarg.get_arg_tag wit_ltac_production_item)), 
                                     Vernacextend.TyTerminal (":=", Vernacextend.TyNonTerminal (
                                                                    Extend.TUentry (Genarg.get_arg_tag wit_tactic), 
                                                                    Vernacextend.TyNil)))))), 
         (let coqpp_body n r e
         (deprecation, locality) = Vernacextend.VtDefault (fun () -> 
# 485 "plugins/ltac/g_ltac.mlg"
   
    let n = Option.default 0 n in
    Tacentries.add_tactic_notation (Locality.make_module_locality locality) n ?deprecation r e;
  
                                   ) in fun n
         r e ~atts -> coqpp_body n r e
         (Attributes.parse Attributes.Notations.(deprecation ++ locality) atts)), Some 
         (fun n r e -> 
# 484 "plugins/ltac/g_ltac.mlg"
    VtSideff ([], VtNow) 
         )))]

let () = Vernacextend.vernac_extend ~command:"VernacPrintLtac" ~classifier:(fun _ -> Vernacextend.classify_as_query) ?entry:None 
         [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Print", 
                                     Vernacextend.TyTerminal ("Ltac", 
                                     Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_reference), 
                                     Vernacextend.TyNil))), (let coqpp_body r
                                                            () = Vernacextend.VtDefault (fun () -> 
                                                                 
# 493 "plugins/ltac/g_ltac.mlg"
    Feedback.msg_notice (Tacintern.print_ltac r) 
                                                                 ) in fun r
                                                            ~atts
                                                            -> coqpp_body r
                                                            (Attributes.unsupported_attributes atts)), None))]

let () = Vernacextend.vernac_extend ~command:"VernacLocateLtac" ~classifier:(fun _ -> Vernacextend.classify_as_query) ?entry:None 
         [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Locate", 
                                     Vernacextend.TyTerminal ("Ltac", 
                                     Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_reference), 
                                     Vernacextend.TyNil))), (let coqpp_body r
                                                            () = Vernacextend.VtDefault (fun () -> 
                                                                 
# 498 "plugins/ltac/g_ltac.mlg"
    Tacentries.print_located_tactic r 
                                                                 ) in fun r
                                                            ~atts
                                                            -> coqpp_body r
                                                            (Attributes.unsupported_attributes atts)), None))]


# 501 "plugins/ltac/g_ltac.mlg"
 

let pr_ltac_ref = Libnames.pr_qualid

let pr_tacdef_body env sigma tacdef_body =
  let id, redef, body =
    match tacdef_body with
    | TacticDefinition ({CAst.v=id}, body) -> Id.print id, false, body
    | TacticRedefinition (id, body) -> pr_ltac_ref id, true, body
  in
  let idl, body =
    match body with
      | Tacexpr.TacFun (idl,b) -> idl,b
      | _ -> [], body in
  id ++
    prlist (function Name.Anonymous -> str " _"
      | Name.Name id -> spc () ++ Id.print id) idl
  ++ (if redef then str" ::=" else str" :=") ++ brk(1,1)
  ++ Pptactic.pr_raw_tactic env sigma body



let (wit_ltac_tacdef_body, ltac_tacdef_body) = Vernacextend.vernac_argument_extend ~name:"ltac_tacdef_body" 
                                               {
                                               Vernacextend.arg_parsing = 
                                               Vernacextend.Arg_alias (tacdef_body);
                                               Vernacextend.arg_printer = fun env sigma -> 
                                               
# 524 "plugins/ltac/g_ltac.mlg"
             pr_tacdef_body env sigma 
                                               ;
                                               }
let _ = (wit_ltac_tacdef_body, ltac_tacdef_body)

let () = Vernacextend.vernac_extend ~command:"VernacDeclareTacticDefinition"  ?entry:None 
         [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Ltac", 
                                     Vernacextend.TyNonTerminal (Extend.TUlist1sep (
                                                                 Extend.TUentry (Genarg.get_arg_tag wit_ltac_tacdef_body), "with"), 
                                     Vernacextend.TyNil)), (let coqpp_body l
                                                           (deprecation, locality) = 
                                                           Vernacextend.VtDefault (fun () -> 
                                                           
# 533 "plugins/ltac/g_ltac.mlg"
        
         Tacentries.register_ltac (Locality.make_module_locality locality) ?deprecation l;
  
                                                           ) in fun l ~atts
                                                           -> coqpp_body l
                                                           (Attributes.parse Attributes.Notations.(deprecation ++ locality) atts)), Some 
         (fun l -> 
# 529 "plugins/ltac/g_ltac.mlg"
                                                                                     
    VtSideff (List.map (function
      | TacticDefinition ({CAst.v=r},_) -> r
      | TacticRedefinition (qid,_) -> qualid_basename qid) l, VtLater)
  
         )))]

let () = Vernacextend.vernac_extend ~command:"VernacPrintLtacs" ~classifier:(fun _ -> Vernacextend.classify_as_query) ?entry:None 
         [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Print", 
                                     Vernacextend.TyTerminal ("Ltac", 
                                     Vernacextend.TyTerminal ("Signatures", 
                                     Vernacextend.TyNil))), (let coqpp_body () = 
                                                            Vernacextend.VtDefault (fun () -> 
                                                            
# 539 "plugins/ltac/g_ltac.mlg"
                                       Tacentries.print_ltacs () 
                                                            ) in fun ~atts
                                                            -> coqpp_body (Attributes.unsupported_attributes atts)), None))]

