let __coq_plugin_name = "ltac2_plugin"
let _ = Mltop.add_known_module __coq_plugin_name

# 13 "user-contrib/Ltac2/g_ltac2.mlg"
 

open Pp
open Util
open Names
open Pcoq
open Attributes
open Constrexpr
open Tac2expr
open Tac2qexpr
open Ltac_plugin

let lk_ident_or_anti =
  Pcoq.Lookahead.(lk_ident <+> (lk_kw "$" >> lk_ident >> check_no_space))

(* lookahead for (x:=t), (?x:=t) and (1:=t) *)
let test_lpar_idnum_coloneq =
  let open Pcoq.Lookahead in
  to_entry "test_lpar_idnum_coloneq" begin
    lk_kw "(" >> (lk_ident_or_anti <+> lk_nat) >> lk_kw ":="
  end

(* lookahead for (x:t), (?x:t) *)
let test_lpar_id_colon =
  let open Pcoq.Lookahead in
  to_entry "test_lpar_id_colon" begin
    lk_kw "(" >> lk_ident_or_anti >> lk_kw ":"
  end

(* Hack to recognize "(x := t)" and "($x := t)" *)
let test_lpar_id_coloneq =
  let open Pcoq.Lookahead in
  to_entry "test_lpar_id_coloneq" begin
    lk_kw "(" >> lk_ident_or_anti >> lk_kw ":="
  end

(* Hack to recognize "(x)" *)
let test_lpar_id_rpar =
  let open Pcoq.Lookahead in
  to_entry "test_lpar_id_rpar" begin
    lk_kw "(" >> lk_ident >> lk_kw ")"
  end

let test_ampersand_ident =
  let open Pcoq.Lookahead in
  to_entry "test_ampersand_ident" begin
    lk_kw "&" >> lk_ident >> check_no_space
  end

let test_dollar_ident =
  let open Pcoq.Lookahead in
  to_entry "test_dollar_ident" begin
    lk_kw "$" >> lk_ident >> check_no_space
  end

let test_ltac1_env =
  let open Pcoq.Lookahead in
  to_entry "test_ltac1_env" begin
    lk_ident_list >> lk_kw "|-"
  end

let tac2expr = Tac2entries.Pltac.tac2expr
let tac2type = Entry.create "tactic:tac2type"
let tac2def_val = Entry.create "tactic:tac2def_val"
let tac2def_typ = Entry.create "tactic:tac2def_typ"
let tac2def_ext = Entry.create "tactic:tac2def_ext"
let tac2def_syn = Entry.create "tactic:tac2def_syn"
let tac2def_mut = Entry.create "tactic:tac2def_mut"
let tac2mode = Entry.create "vernac:ltac2_command"

let ltac1_expr = Pltac.tactic_expr
let tac2expr_in_env = Tac2entries.Pltac.tac2expr_in_env

let inj_wit wit loc x = CAst.make ~loc @@ CTacExt (wit, x)
let inj_open_constr loc c = inj_wit Tac2quote.wit_open_constr loc c
let inj_pattern loc c = inj_wit Tac2quote.wit_pattern loc c
let inj_reference loc c = inj_wit Tac2quote.wit_reference loc c
let inj_ltac1 loc e = inj_wit Tac2quote.wit_ltac1 loc e
let inj_ltac1val loc e = inj_wit Tac2quote.wit_ltac1val loc e

let pattern_of_qualid qid =
  if Tac2env.is_constructor qid then CAst.make ?loc:qid.CAst.loc @@ CPatRef (RelId qid, [])
  else
    let open Libnames in
    if qualid_is_ident qid then CAst.make ?loc:qid.CAst.loc @@ CPatVar (Name (qualid_basename qid))
    else
      CErrors.user_err ?loc:qid.CAst.loc (Pp.str "Syntax error")



let _ = let tac2pat = Pcoq.Entry.make "tac2pat"
        and atomic_tac2pat = Pcoq.Entry.make "atomic_tac2pat"
        and branches = Pcoq.Entry.make "branches"
        and branch = Pcoq.Entry.make "branch"
        and rec_flag = Pcoq.Entry.make "rec_flag"
        and mut_flag = Pcoq.Entry.make "mut_flag"
        and typ_param = Pcoq.Entry.make "typ_param"
        and tactic_atom = Pcoq.Entry.make "tactic_atom"
        and ltac1_expr_in_env = Pcoq.Entry.make "ltac1_expr_in_env"
        and let_clause = Pcoq.Entry.make "let_clause"
        and let_binder = Pcoq.Entry.make "let_binder"
        and locident = Pcoq.Entry.make "locident"
        and binder = Pcoq.Entry.make "binder"
        and input_fun = Pcoq.Entry.make "input_fun"
        and tac2def_body = Pcoq.Entry.make "tac2def_body"
        and tac2typ_knd = Pcoq.Entry.make "tac2typ_knd"
        and tac2alg_constructors = Pcoq.Entry.make "tac2alg_constructors"
        and tac2alg_constructor = Pcoq.Entry.make "tac2alg_constructor"
        and tac2rec_fields = Pcoq.Entry.make "tac2rec_fields"
        and tac2rec_field = Pcoq.Entry.make "tac2rec_field"
        and tac2rec_fieldexprs = Pcoq.Entry.make "tac2rec_fieldexprs"
        and tac2rec_fieldexpr = Pcoq.Entry.make "tac2rec_fieldexpr"
        and tac2typ_prm = Pcoq.Entry.make "tac2typ_prm"
        and tac2typ_def = Pcoq.Entry.make "tac2typ_def"
        and tac2type_body = Pcoq.Entry.make "tac2type_body"
        and syn_node = Pcoq.Entry.make "syn_node"
        and sexpr = Pcoq.Entry.make "sexpr"
        and syn_level = Pcoq.Entry.make "syn_level"
        and lident = Pcoq.Entry.make "lident"
        and globref = Pcoq.Entry.make "globref"
        in
        let () =
        Pcoq.grammar_extend tac2pat
        { Pcoq.pos=None; data=[(Some ("1"), Some (Gramlib.Gramext.LeftA),
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm tac2pat)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("::")))))
                                                ((Pcoq.Symbol.nterm tac2pat)))
                                (fun p2 _ p1 loc -> 
# 116 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CPatRef (AbsKn (Other Tac2core.Core.c_cons), [p1; p2])
                                                    );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ _ loc -> 
# 114 "user-contrib/Ltac2/g_ltac2.mlg"
                      CAst.make ~loc @@ CPatRef (AbsKn (Other Tac2core.Core.c_nil), []) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.qualid)))
                               (fun qid loc -> 
# 113 "user-contrib/Ltac2/g_ltac2.mlg"
                               pattern_of_qualid qid 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm Prim.qualid)))
                                               ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterml tac2pat ("0"))))))
                               (fun pl qid loc -> 
# 108 "user-contrib/Ltac2/g_ltac2.mlg"
                                                            
        if Tac2env.is_constructor qid then
          CAst.make ~loc @@ CPatRef (RelId qid, pl)
        else
          CErrors.user_err ~loc (Pp.str "Syntax error") 
                                                  )]);
                              (Some ("0"), None,
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm atomic_tac2pat)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ p _ loc -> 
# 122 "user-contrib/Ltac2/g_ltac2.mlg"
                                          p 
                                                 );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.nterm Prim.qualid)))
                              (fun qid loc -> 
# 121 "user-contrib/Ltac2/g_ltac2.mlg"
                               pattern_of_qualid qid 
                                              );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("()")))))
                              (fun _ loc -> 
# 120 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc @@ CPatRef (AbsKn (Tuple 0), []) 
                                            );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                              (fun _ loc -> 
# 119 "user-contrib/Ltac2/g_ltac2.mlg"
                 CAst.make ~loc @@ CPatVar Anonymous 
                                            )])]}
        in let () =
        Pcoq.grammar_extend atomic_tac2pat
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm tac2pat)))
                                (fun p loc -> 
# 133 "user-contrib/Ltac2/g_ltac2.mlg"
                         p 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2pat)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))))
                                               ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm tac2pat)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                               (fun pl _ p loc -> 
# 131 "user-contrib/Ltac2/g_ltac2.mlg"
          let pl = p :: pl in
          CAst.make ~loc @@ CPatRef (AbsKn (Tuple (List.length pl)), pl) 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2pat)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                               ((Pcoq.Symbol.nterm tac2type)))
                               (fun t _ p loc -> 
# 129 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CPatCnv (p, t) 
                                                 );
                               Pcoq.Production.make (Pcoq.Rule.stop)
                               (fun loc -> 
# 127 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CPatRef (AbsKn (Tuple 0), []) 
                                           )])]}
        in let () =
        Pcoq.grammar_extend tac2expr
        { Pcoq.pos=None; data=[(Some ("6"), Some (Gramlib.Gramext.RightA),
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                (Pcoq.Symbol.self))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                                                (Pcoq.Symbol.self))
                                (fun e2 _ e1 loc -> 
# 138 "user-contrib/Ltac2/g_ltac2.mlg"
                                         CAst.make ~loc @@ CTacSeq (e1, e2) 
                                                    )]);
                              (Some ("5"), None,
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("match")))))
                                                               ((Pcoq.Symbol.nterml tac2expr ("5"))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                               ((Pcoq.Symbol.nterm branches)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("end")))))
                               (fun _ bl _ e _ loc -> 
# 147 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacCse (e, bl) 
                                                      );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("let")))))
                                                              ((Pcoq.Symbol.nterm rec_flag)))
                                                              ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm let_clause)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))) false)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("in")))))
                                              ((Pcoq.Symbol.nterml tac2expr ("6"))))
                              (fun e _ lc isrec _ loc -> 
# 145 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacLet (isrec, lc, e) 
                                                         );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("fun")))))
                                                              ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm input_fun)))))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                              ((Pcoq.Symbol.nterml tac2expr ("6"))))
                              (fun body _ it _ loc -> 
# 141 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacFun (it, body) 
                                                      )]);
                              (Some ("4"), Some (Gramlib.Gramext.LeftA), []);
                              (Some ("3"), None,
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               (Pcoq.Symbol.self))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))))
                                               ((Pcoq.Symbol.list1sep (Pcoq.Symbol.next) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                               (fun el _ e0 loc -> 
# 151 "user-contrib/Ltac2/g_ltac2.mlg"
          let el = e0 :: el in
          CAst.make ~loc @@ CTacApp (CAst.make ~loc @@ CTacCst (AbsKn (Tuple (List.length el))), el) 
                                                   )]);
                              (Some ("2"), Some (Gramlib.Gramext.RightA),
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("::")))))
                                               ((Pcoq.Symbol.nterm tac2expr)))
                               (fun e2 _ e1 loc -> 
# 155 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacApp (CAst.make ~loc @@ CTacCst (AbsKn (Other Tac2core.Core.c_cons)), [e1; e2]) 
                                                   )]);
                              (Some ("1"), Some (Gramlib.Gramext.LeftA),
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               (Pcoq.Symbol.self))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (".(")))))
                                                               ((Pcoq.Symbol.nterm Prim.qualid)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                               ((Pcoq.Symbol.nterml tac2expr ("5"))))
                               (fun r _ _ qid _ e loc -> 
# 163 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacSet (e, RelId qid, r) 
                                                         );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              (Pcoq.Symbol.self))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (".(")))))
                                                              ((Pcoq.Symbol.nterm Prim.qualid)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                              (fun _ qid _ e loc -> 
# 161 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacPrj (e, RelId qid) 
                                                    );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.nterm tac2expr)))
                                              ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterml tac2expr ("0"))))))
                              (fun el e loc -> 
# 159 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacApp (e, el) 
                                               )]);
                              (Some ("0"), None,
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tactic_atom)))
                               (fun a loc -> 
# 176 "user-contrib/Ltac2/g_ltac2.mlg"
                             a 
                                             );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("{")))))
                                                              ((Pcoq.Symbol.nterm tac2rec_fieldexprs)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("}")))))
                              (fun _ a _ loc -> 
# 175 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacRec a 
                                                );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                              ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterml tac2expr ("5"))) ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))) false)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                              (fun _ a _ loc -> 
# 173 "user-contrib/Ltac2/g_ltac2.mlg"
          Tac2quote.of_list ~loc (fun x -> x) a 
                                                );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                              (fun _ _ loc -> 
# 171 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacCst (AbsKn (Tuple 0)) 
                                              );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("()")))))
                              (fun _ loc -> 
# 169 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacCst (AbsKn (Tuple 0)) 
                                            );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                              (Pcoq.Symbol.self))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                              ((Pcoq.Symbol.nterm tac2type)))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                              (fun _ t _ a _ loc -> 
# 167 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ CTacCnv (a, t) 
                                                    );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                              (Pcoq.Symbol.self))
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                              (fun _ a _ loc -> 
# 165 "user-contrib/Ltac2/g_ltac2.mlg"
                                a 
                                                )])]}
        in let () =
        Pcoq.grammar_extend branches
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm branch)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                (fun bl loc -> 
# 182 "user-contrib/Ltac2/g_ltac2.mlg"
                                     bl 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm branch)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun bl _ loc -> 
# 181 "user-contrib/Ltac2/g_ltac2.mlg"
                                          bl 
                                                );
                               Pcoq.Production.make (Pcoq.Rule.stop)
                               (fun loc -> 
# 180 "user-contrib/Ltac2/g_ltac2.mlg"
           [] 
                                           )])]}
        in let () =
        Pcoq.grammar_extend branch
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterml tac2pat ("1"))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                                ((Pcoq.Symbol.nterml tac2expr ("6"))))
                                (fun e _ pat loc -> 
# 186 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                 (pat, e) 
                                                    )])]}
        in let () =
        Pcoq.grammar_extend rec_flag
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 190 "user-contrib/Ltac2/g_ltac2.mlg"
             false 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("rec"))))))
                               (fun _ loc -> 
# 189 "user-contrib/Ltac2/g_ltac2.mlg"
                         true 
                                             )])]}
        in let () =
        Pcoq.grammar_extend mut_flag
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 194 "user-contrib/Ltac2/g_ltac2.mlg"
             false 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("mutable"))))))
                               (fun _ loc -> 
# 193 "user-contrib/Ltac2/g_ltac2.mlg"
                             true 
                                             )])]}
        in let () =
        Pcoq.grammar_extend typ_param
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("'")))))
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id _ loc -> 
# 197 "user-contrib/Ltac2/g_ltac2.mlg"
                                  id 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend tactic_atom
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("ltac1val"))))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                ((Pcoq.Symbol.nterm ltac1_expr_in_env)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                (fun _ qid _ _ _ loc -> 
# 216 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                      inj_ltac1val loc qid 
                                                        );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("ltac1"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm ltac1_expr_in_env)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ qid _ _ _ loc -> 
# 215 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                   inj_ltac1 loc qid 
                                                       );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("reference"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm globref)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ c _ _ _ loc -> 
# 214 "user-contrib/Ltac2/g_ltac2.mlg"
                                                           inj_reference loc c 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("pattern"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr_pattern)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ c _ _ _ loc -> 
# 213 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                        inj_pattern loc c 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("ident"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm lident)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ c _ _ _ loc -> 
# 212 "user-contrib/Ltac2/g_ltac2.mlg"
                                                      Tac2quote.of_ident c 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("open_constr"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ c _ _ _ loc -> 
# 211 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                    Tac2quote.of_open_constr c 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("constr"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ c _ _ _ loc -> 
# 210 "user-contrib/Ltac2/g_ltac2.mlg"
                                                               Tac2quote.of_constr c 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("'")))))
                                               ((Pcoq.Symbol.nterm Constr.constr)))
                               (fun c _ loc -> 
# 209 "user-contrib/Ltac2/g_ltac2.mlg"
                                    inj_open_constr loc c 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("&")))))
                                               ((Pcoq.Symbol.nterm lident)))
                               (fun id _ loc -> 
# 208 "user-contrib/Ltac2/g_ltac2.mlg"
                              Tac2quote.of_hyp ~loc id 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("@")))))
                                               ((Pcoq.Symbol.nterm Prim.ident)))
                               (fun id _ loc -> 
# 207 "user-contrib/Ltac2/g_ltac2.mlg"
                                  Tac2quote.of_ident (CAst.make ~loc id) 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.qualid)))
                               (fun qid loc -> 
# 203 "user-contrib/Ltac2/g_ltac2.mlg"
        if Tac2env.is_constructor qid then
          CAst.make ~loc @@ CTacCst (RelId qid)
        else
          CAst.make ~loc @@ CTacRef (RelId qid) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.string)))
                               (fun s loc -> 
# 201 "user-contrib/Ltac2/g_ltac2.mlg"
                             CAst.make ~loc @@ CTacAtm (AtmStr s) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.integer)))
                               (fun n loc -> 
# 200 "user-contrib/Ltac2/g_ltac2.mlg"
                              CAst.make ~loc @@ CTacAtm (AtmInt n) 
                                             )])]}
        in let () =
        Pcoq.grammar_extend ltac1_expr_in_env
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm ltac1_expr)))
                                (fun e loc -> 
# 221 "user-contrib/Ltac2/g_ltac2.mlg"
                            [], e 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm test_ltac1_env)))
                                                               ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm locident))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                               ((Pcoq.Symbol.nterm ltac1_expr)))
                               (fun e _ ids _ loc -> 
# 220 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                        ids, e 
                                                     )])]}
        in let () =
        Pcoq.grammar_extend tac2expr_in_env
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm tac2expr)))
                                (fun tac loc -> 
# 233 "user-contrib/Ltac2/g_ltac2.mlg"
                            [], tac 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm test_ltac1_env)))
                                                               ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm locident))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                               ((Pcoq.Symbol.nterm tac2expr)))
                               (fun e _ ids _ loc -> 
# 226 "user-contrib/Ltac2/g_ltac2.mlg"
        let check { CAst.v = id; CAst.loc = loc } =
          if Tac2env.is_constructor (Libnames.qualid_of_ident ?loc id) then
            CErrors.user_err ?loc Pp.(str "Invalid bound Ltac2 identifier " ++ Id.print id)
        in
        let () = List.iter check ids in
        (ids, e)
      
                                                     )])]}
        in let () =
        Pcoq.grammar_extend let_clause
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm let_binder)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                ((Pcoq.Symbol.nterm tac2expr)))
                                (fun te _ binder loc -> 
# 238 "user-contrib/Ltac2/g_ltac2.mlg"
        let (pat, fn) = binder in
        let te = match fn with
        | None -> te
        | Some args -> CAst.make ~loc @@ CTacFun (args, te)
        in
        (pat, te) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend let_binder
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm input_fun)))))
                                (fun pats loc -> 
# 248 "user-contrib/Ltac2/g_ltac2.mlg"
        match pats with
        | [{CAst.v=CPatVar _} as pat] -> (pat, None)
        | ({CAst.v=CPatVar (Name id)} as pat) :: args -> (pat, Some args)
        | [pat] -> (pat, None)
        | _ -> CErrors.user_err ~loc (str "Invalid pattern") 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend tac2type
        { Pcoq.pos=None; data=[(Some ("5"), Some (Gramlib.Gramext.RightA),
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm tac2type)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("->")))))
                                                ((Pcoq.Symbol.nterm tac2type)))
                                (fun t2 _ t1 loc -> 
# 257 "user-contrib/Ltac2/g_ltac2.mlg"
                                                CAst.make ~loc @@ CTypArrow (t1, t2) 
                                                    )]);
                              (Some ("2"), None,
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2type)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("*")))))
                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterml tac2type ("1"))) ((Pcoq.Symbol.token (Tok.PKEYWORD ("*")))) false)))
                               (fun tl _ t loc -> 
# 260 "user-contrib/Ltac2/g_ltac2.mlg"
        let tl = t :: tl in
        CAst.make ~loc @@ CTypRef (AbsKn (Tuple (List.length tl)), tl) 
                                                  )]);
                              (Some ("1"), Some (Gramlib.Gramext.LeftA),
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               (Pcoq.Symbol.self))
                                               ((Pcoq.Symbol.nterm Prim.qualid)))
                               (fun qid t loc -> 
# 263 "user-contrib/Ltac2/g_ltac2.mlg"
                                         CAst.make ~loc @@ CTypRef (RelId qid, [t]) 
                                                 )]);
                              (Some ("0"), None,
                              [Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.qualid)))
                               (fun qid loc -> 
# 273 "user-contrib/Ltac2/g_ltac2.mlg"
                               CAst.make ~loc @@ CTypRef (RelId qid, []) 
                                               );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                              (fun _ loc -> 
# 272 "user-contrib/Ltac2/g_ltac2.mlg"
                 CAst.make ~loc @@ CTypVar Anonymous 
                                            );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.stop)
                                              ((Pcoq.Symbol.nterm typ_param)))
                              (fun id loc -> 
# 271 "user-contrib/Ltac2/g_ltac2.mlg"
                            CAst.make ~loc @@ CTypVar (Name id) 
                                             );
                              Pcoq.Production.make
                              (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                              (Pcoq.Rule.next 
                                                              (Pcoq.Rule.stop)
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                              ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterml tac2type ("5"))) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                              ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                              ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm Prim.qualid))))
                              (fun qid _ p _ loc -> 
# 266 "user-contrib/Ltac2/g_ltac2.mlg"
          match p, qid with
          | [t], None -> t
          | _, None -> CErrors.user_err ~loc (Pp.str "Syntax error")
          | ts, Some qid -> CAst.make ~loc @@ CTypRef (RelId qid, p)
        
                                                    )])]}
        in let () =
        Pcoq.grammar_extend locident
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id loc -> 
# 277 "user-contrib/Ltac2/g_ltac2.mlg"
                             CAst.make ~loc id 
                                               )])]}
        in let () =
        Pcoq.grammar_extend binder
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun l loc -> 
# 281 "user-contrib/Ltac2/g_ltac2.mlg"
                            CAst.make ~loc (Name l) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                               (fun _ loc -> 
# 280 "user-contrib/Ltac2/g_ltac2.mlg"
                 CAst.make ~loc Anonymous 
                                             )])]}
        in let () =
        Pcoq.grammar_extend input_fun
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterml tac2pat ("0"))))
                                (fun b loc -> 
# 284 "user-contrib/Ltac2/g_ltac2.mlg"
                                   b 
                                              )])]}
        in let () =
        Pcoq.grammar_extend tac2def_body
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm binder)))
                                                                ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm input_fun))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                ((Pcoq.Symbol.nterm tac2expr)))
                                (fun e _ it name loc -> 
# 288 "user-contrib/Ltac2/g_ltac2.mlg"
        let e = if List.is_empty it then e else CAst.make ~loc @@ CTacFun (it, e) in
        (name, e) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend tac2def_val
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm mut_flag)))
                                                                ((Pcoq.Symbol.nterm rec_flag)))
                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm tac2def_body)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))) false)))
                                (fun l isrec mut loc -> 
# 294 "user-contrib/Ltac2/g_ltac2.mlg"
          StrVal (mut, isrec, l) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend tac2def_mut
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("Set")))))
                                                                ((Pcoq.Symbol.nterm Prim.qualid)))
                                                                ((Pcoq.Symbol.opt (Pcoq.Symbol.rules 
                                                                [Pcoq.Rules.make 
                                                                (Pcoq.Rule.next_norec 
                                                                (Pcoq.Rule.next_norec 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("as")))))
                                                                ((Pcoq.Symbol.nterm locident)))
                                                                (fun id _
                                                                loc -> 
                                                                
# 298 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                       id 
                                                                )]))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                ((Pcoq.Symbol.nterm tac2expr)))
                                (fun e _ old qid _ loc -> 
# 298 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                                                       StrMut (qid, old, e) 
                                                          )])]}
        in let () =
        Pcoq.grammar_extend tac2typ_knd
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("{")))))
                                                                ((Pcoq.Symbol.nterm tac2rec_fields)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("}")))))
                                (fun _ t _ loc -> 
# 304 "user-contrib/Ltac2/g_ltac2.mlg"
                                         CTydRec t 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.nterm tac2alg_constructors)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ t _ loc -> 
# 303 "user-contrib/Ltac2/g_ltac2.mlg"
                                                CTydAlg t 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("..")))))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ _ _ loc -> 
# 302 "user-contrib/Ltac2/g_ltac2.mlg"
                            CTydOpn 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tac2type)))
                               (fun t loc -> 
# 301 "user-contrib/Ltac2/g_ltac2.mlg"
                          CTydDef (Some t) 
                                             )])]}
        in let () =
        Pcoq.grammar_extend tac2alg_constructors
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm tac2alg_constructor)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                (fun cs loc -> 
# 308 "user-contrib/Ltac2/g_ltac2.mlg"
                                                    cs 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm tac2alg_constructor)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun cs _ loc -> 
# 307 "user-contrib/Ltac2/g_ltac2.mlg"
                                                         cs 
                                                )])]}
        in let () =
        Pcoq.grammar_extend tac2alg_constructor
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm tac2type)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                (fun _ args _ c loc -> 
# 312 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                    (c, args) 
                                                       );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.ident)))
                               (fun c loc -> 
# 311 "user-contrib/Ltac2/g_ltac2.mlg"
                            (c, []) 
                                             )])]}
        in let () =
        Pcoq.grammar_extend tac2rec_fields
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 318 "user-contrib/Ltac2/g_ltac2.mlg"
             [] 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tac2rec_field)))
                               (fun f loc -> 
# 317 "user-contrib/Ltac2/g_ltac2.mlg"
                               [f] 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2rec_field)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                               (fun _ f loc -> 
# 316 "user-contrib/Ltac2/g_ltac2.mlg"
                                    [f] 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2rec_field)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                                               ((Pcoq.Symbol.nterm tac2rec_fields)))
                               (fun l _ f loc -> 
# 315 "user-contrib/Ltac2/g_ltac2.mlg"
                                                        f :: l 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend tac2rec_field
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm mut_flag)))
                                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                ((Pcoq.Symbol.nterm tac2type)))
                                (fun t _ id mut loc -> 
# 321 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                (id, mut, t) 
                                                       )])]}
        in let () =
        Pcoq.grammar_extend tac2rec_fieldexprs
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 327 "user-contrib/Ltac2/g_ltac2.mlg"
             [] 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tac2rec_fieldexpr)))
                               (fun f loc -> 
# 326 "user-contrib/Ltac2/g_ltac2.mlg"
                                  [f] 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2rec_fieldexpr)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                               (fun _ f loc -> 
# 325 "user-contrib/Ltac2/g_ltac2.mlg"
                                        [f] 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2rec_fieldexpr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (";")))))
                                               ((Pcoq.Symbol.nterm tac2rec_fieldexprs)))
                               (fun l _ f loc -> 
# 324 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                f :: l 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend tac2rec_fieldexpr
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Prim.qualid)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                ((Pcoq.Symbol.nterml tac2expr ("1"))))
                                (fun e _ qid loc -> 
# 330 "user-contrib/Ltac2/g_ltac2.mlg"
                                                             RelId qid, e 
                                                    )])]}
        in let () =
        Pcoq.grammar_extend tac2typ_prm
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.rules 
                                                                [Pcoq.Rules.make 
                                                                (Pcoq.Rule.next_norec 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm typ_param)))
                                                                (fun id
                                                                loc -> 
                                                                
# 335 "user-contrib/Ltac2/g_ltac2.mlg"
                                               CAst.make ~loc id 
                                                                )])) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                (fun _ ids _ loc -> 
# 335 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                                       ids 
                                                    );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm typ_param)))
                               (fun id loc -> 
# 334 "user-contrib/Ltac2/g_ltac2.mlg"
                            [CAst.make ~loc id] 
                                              );
                               Pcoq.Production.make (Pcoq.Rule.stop)
                               (fun loc -> 
# 333 "user-contrib/Ltac2/g_ltac2.mlg"
             [] 
                                           )])]}
        in let () =
        Pcoq.grammar_extend tac2typ_def
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm tac2typ_prm)))
                                                                ((Pcoq.Symbol.nterm Prim.qualid)))
                                                ((Pcoq.Symbol.nterm tac2type_body)))
                                (fun b id prm loc -> 
# 339 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                    let (r, e) = b in (id, r, (prm, e)) 
                                                     )])]}
        in let () =
        Pcoq.grammar_extend tac2type_body
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("::=")))))
                                                ((Pcoq.Symbol.nterm tac2typ_knd)))
                                (fun e _ loc -> 
# 344 "user-contrib/Ltac2/g_ltac2.mlg"
                                    true, e 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                               ((Pcoq.Symbol.nterm tac2typ_knd)))
                               (fun e _ loc -> 
# 343 "user-contrib/Ltac2/g_ltac2.mlg"
                                   false, e 
                                               );
                               Pcoq.Production.make (Pcoq.Rule.stop)
                               (fun loc -> 
# 342 "user-contrib/Ltac2/g_ltac2.mlg"
             false, CTydDef None 
                                           )])]}
        in let () =
        Pcoq.grammar_extend tac2def_typ
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("Type")))))
                                                                ((Pcoq.Symbol.nterm rec_flag)))
                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm tac2typ_def)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))) false)))
                                (fun l isrec _ loc -> 
# 349 "user-contrib/Ltac2/g_ltac2.mlg"
          StrTyp (isrec, l) 
                                                      )])]}
        in let () =
        Pcoq.grammar_extend tac2def_ext
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("@")))))
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("external"))))))
                                                                ((Pcoq.Symbol.nterm locident)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                                ((Pcoq.Symbol.nterml tac2type ("5"))))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                                ((Pcoq.Symbol.nterm Prim.string)))
                                                ((Pcoq.Symbol.nterm Prim.string)))
                                (fun name plugin _ t _ id _ _ loc -> 
# 355 "user-contrib/Ltac2/g_ltac2.mlg"
        let ml = { mltac_plugin = plugin; mltac_tactic = name } in
        StrPrm (id, t, ml) 
                                                                    )])]}
        in let () =
        Pcoq.grammar_extend syn_node
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id loc -> 
# 361 "user-contrib/Ltac2/g_ltac2.mlg"
                             CAst.make ~loc (Some id) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                               (fun _ loc -> 
# 360 "user-contrib/Ltac2/g_ltac2.mlg"
                 CAst.make ~loc None 
                                             )])]}
        in let () =
        Pcoq.grammar_extend sexpr
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm syn_node)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm sexpr)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                (fun _ tok _ id loc -> 
# 369 "user-contrib/Ltac2/g_ltac2.mlg"
          SexprRec (loc, id, tok) 
                                                       );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm syn_node)))
                               (fun id loc -> 
# 367 "user-contrib/Ltac2/g_ltac2.mlg"
                           SexprRec (loc, id, []) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.integer)))
                               (fun n loc -> 
# 366 "user-contrib/Ltac2/g_ltac2.mlg"
                              SexprInt (CAst.make ~loc n) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.string)))
                               (fun s loc -> 
# 365 "user-contrib/Ltac2/g_ltac2.mlg"
                             SexprStr (CAst.make ~loc s) 
                                             )])]}
        in let () =
        Pcoq.grammar_extend syn_level
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                ((Pcoq.Symbol.nterm Prim.integer)))
                                (fun n _ loc -> 
# 374 "user-contrib/Ltac2/g_ltac2.mlg"
                                   Some n 
                                                );
                               Pcoq.Production.make (Pcoq.Rule.stop)
                               (fun loc -> 
# 373 "user-contrib/Ltac2/g_ltac2.mlg"
             None 
                                           )])]}
        in let () =
        Pcoq.grammar_extend tac2def_syn
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("Notation")))))
                                                                ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm sexpr)))))
                                                                ((Pcoq.Symbol.nterm syn_level)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                ((Pcoq.Symbol.nterm tac2expr)))
                                (fun e _ n toks _ loc -> 
# 380 "user-contrib/Ltac2/g_ltac2.mlg"
          StrSyn (toks, n, e) 
                                                         )])]}
        in let () =
        Pcoq.grammar_extend lident
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id loc -> 
# 384 "user-contrib/Ltac2/g_ltac2.mlg"
                             CAst.make ~loc id 
                                               )])]}
        in let () =
        Pcoq.grammar_extend globref
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Prim.qualid)))
                                (fun qid loc -> 
# 388 "user-contrib/Ltac2/g_ltac2.mlg"
                               CAst.make ~loc @@ QReference qid 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("&")))))
                                               ((Pcoq.Symbol.nterm Prim.ident)))
                               (fun id _ loc -> 
# 387 "user-contrib/Ltac2/g_ltac2.mlg"
                                  CAst.make ~loc (QHypothesis id) 
                                                )])]}
        in ()


# 395 "user-contrib/Ltac2/g_ltac2.mlg"
 

open Tac2entries.Pltac

let loc_of_ne_list l = Loc.merge_opt (fst (List.hd l)) (fst (List.last l))



let _ = let anti = Pcoq.Entry.make "anti"
        and ident_or_anti = Pcoq.Entry.make "ident_or_anti"
        and lident = Pcoq.Entry.make "lident"
        and lnatural = Pcoq.Entry.make "lnatural"
        and qhyp = Pcoq.Entry.make "qhyp"
        and simple_binding = Pcoq.Entry.make "simple_binding"
        and bindings = Pcoq.Entry.make "bindings"
        and intropatterns = Pcoq.Entry.make "intropatterns"
        and or_and_intropattern = Pcoq.Entry.make "or_and_intropattern"
        and equality_intropattern = Pcoq.Entry.make "equality_intropattern"
        and naming_intropattern = Pcoq.Entry.make "naming_intropattern"
        and nonsimple_intropattern = Pcoq.Entry.make "nonsimple_intropattern"
        and simple_intropattern = Pcoq.Entry.make "simple_intropattern"
        and simple_intropattern_closed =
          Pcoq.Entry.make "simple_intropattern_closed"
        and nat_or_anti = Pcoq.Entry.make "nat_or_anti"
        and eqn_ipat = Pcoq.Entry.make "eqn_ipat"
        and with_bindings = Pcoq.Entry.make "with_bindings"
        and constr_with_bindings = Pcoq.Entry.make "constr_with_bindings"
        and destruction_arg = Pcoq.Entry.make "destruction_arg"
        and as_or_and_ipat = Pcoq.Entry.make "as_or_and_ipat"
        and occs_nums = Pcoq.Entry.make "occs_nums"
        and occs = Pcoq.Entry.make "occs"
        and hypident = Pcoq.Entry.make "hypident"
        and hypident_occ = Pcoq.Entry.make "hypident_occ"
        and in_clause = Pcoq.Entry.make "in_clause"
        and clause = Pcoq.Entry.make "clause"
        and concl_occ = Pcoq.Entry.make "concl_occ"
        and induction_clause = Pcoq.Entry.make "induction_clause"
        and conversion = Pcoq.Entry.make "conversion"
        and ltac2_orient = Pcoq.Entry.make "ltac2_orient"
        and rewriter = Pcoq.Entry.make "rewriter"
        and oriented_rewriter = Pcoq.Entry.make "oriented_rewriter"
        and tactic_then_last = Pcoq.Entry.make "tactic_then_last"
        and tactic_then_gen = Pcoq.Entry.make "tactic_then_gen"
        and red_flag = Pcoq.Entry.make "red_flag"
        and refglobal = Pcoq.Entry.make "refglobal"
        and refglobals = Pcoq.Entry.make "refglobals"
        and delta_flag = Pcoq.Entry.make "delta_flag"
        and strategy_flag = Pcoq.Entry.make "strategy_flag"
        and hintdb = Pcoq.Entry.make "hintdb"
        and match_pattern = Pcoq.Entry.make "match_pattern"
        and match_rule = Pcoq.Entry.make "match_rule"
        and match_list = Pcoq.Entry.make "match_list"
        and gmatch_hyp_pattern = Pcoq.Entry.make "gmatch_hyp_pattern"
        and gmatch_pattern = Pcoq.Entry.make "gmatch_pattern"
        and gmatch_rule = Pcoq.Entry.make "gmatch_rule"
        and gmatch_list = Pcoq.Entry.make "gmatch_list"
        and move_location = Pcoq.Entry.make "move_location"
        and as_name = Pcoq.Entry.make "as_name"
        and pose = Pcoq.Entry.make "pose"
        and as_ipat = Pcoq.Entry.make "as_ipat"
        and by_tactic = Pcoq.Entry.make "by_tactic"
        and assertion = Pcoq.Entry.make "assertion"
        in
        let () =
        Pcoq.grammar_extend anti
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("$")))))
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id _ loc -> 
# 409 "user-contrib/Ltac2/g_ltac2.mlg"
                                  QAnti (CAst.make ~loc id) 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend ident_or_anti
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("$")))))
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id _ loc -> 
# 413 "user-contrib/Ltac2/g_ltac2.mlg"
                                  QAnti (CAst.make ~loc id) 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm lident)))
                               (fun id loc -> 
# 412 "user-contrib/Ltac2/g_ltac2.mlg"
                         QExpr id 
                                              )])]}
        in let () =
        Pcoq.grammar_extend lident
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id loc -> 
# 417 "user-contrib/Ltac2/g_ltac2.mlg"
                             CAst.make ~loc id 
                                               )])]}
        in let () =
        Pcoq.grammar_extend lnatural
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Prim.natural)))
                                (fun n loc -> 
# 420 "user-contrib/Ltac2/g_ltac2.mlg"
                              CAst.make ~loc n 
                                              )])]}
        in let () =
        Pcoq.grammar_extend q_ident
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm ident_or_anti)))
                                (fun id loc -> 
# 423 "user-contrib/Ltac2/g_ltac2.mlg"
                                id 
                                               )])]}
        in let () =
        Pcoq.grammar_extend qhyp
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm lident)))
                                (fun id loc -> 
# 428 "user-contrib/Ltac2/g_ltac2.mlg"
                         QExpr (CAst.make ~loc @@ QNamedHyp id) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm lnatural)))
                               (fun n loc -> 
# 427 "user-contrib/Ltac2/g_ltac2.mlg"
                          QExpr (CAst.make ~loc @@ QAnonHyp n) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm anti)))
                               (fun x loc -> 
# 426 "user-contrib/Ltac2/g_ltac2.mlg"
                      x 
                                             )])]}
        in let () =
        Pcoq.grammar_extend simple_binding
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                ((Pcoq.Symbol.nterm qhyp)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                                ((Pcoq.Symbol.nterm Constr.lconstr)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                (fun _ c _ h _ loc -> 
# 433 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc (h, c) 
                                                      )])]}
        in let () =
        Pcoq.grammar_extend bindings
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm Constr.constr)))))
                                (fun bl loc -> 
# 440 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QImplicitBindings bl 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm test_lpar_idnum_coloneq)))
                                               ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm simple_binding)))))
                               (fun bl _ loc -> 
# 438 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QExplicitBindings bl 
                                                )])]}
        in let () =
        Pcoq.grammar_extend q_bindings
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm bindings)))
                                (fun bl loc -> 
# 444 "user-contrib/Ltac2/g_ltac2.mlg"
                           bl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend q_with_bindings
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm with_bindings)))
                                (fun bl loc -> 
# 447 "user-contrib/Ltac2/g_ltac2.mlg"
                                bl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend intropatterns
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm nonsimple_intropattern))))
                                (fun l loc -> 
# 450 "user-contrib/Ltac2/g_ltac2.mlg"
                                              CAst.make ~loc l 
                                              )])]}
        in let () =
        Pcoq.grammar_extend or_and_intropattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                ((Pcoq.Symbol.nterm simple_intropattern)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("&")))))
                                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm simple_intropattern)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("&")))) false)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                (fun _ tc _ si _ loc -> 
# 465 "user-contrib/Ltac2/g_ltac2.mlg"
          let rec pairify = function
            | ([]|[_]|[_;_]) as l -> CAst.make ~loc l
            | t::q ->
              let q =
                CAst.make ~loc @@
                  QIntroAction (CAst.make ~loc @@
                    QIntroOrAndPattern (CAst.make ~loc @@
                      QIntroAndPattern (pairify q)))
              in
              CAst.make ~loc [t; q]
          in CAst.make ~loc @@ QIntroAndPattern (pairify (si::tc)) 
                                                        );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm simple_intropattern)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))))
                                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm simple_intropattern)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ tc _ si _ loc -> 
# 461 "user-contrib/Ltac2/g_ltac2.mlg"
               CAst.make ~loc @@ QIntroAndPattern (CAst.make ~loc (si::tc)) 
                                                       );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm simple_intropattern)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ si _ loc -> 
# 458 "user-contrib/Ltac2/g_ltac2.mlg"
                                                CAst.make ~loc @@ QIntroAndPattern (CAst.make ~loc [si]) 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("()")))))
                               (fun _ loc -> 
# 457 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc @@ QIntroAndPattern (CAst.make ~loc []) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm intropatterns)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ tc _ loc -> 
# 456 "user-contrib/Ltac2/g_ltac2.mlg"
                                                        CAst.make ~loc @@ QIntroOrPattern tc 
                                                  )])]}
        in let () =
        Pcoq.grammar_extend equality_intropattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("[=")))))
                                                                ((Pcoq.Symbol.nterm intropatterns)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                                (fun _ tc _ loc -> 
# 480 "user-contrib/Ltac2/g_ltac2.mlg"
                                           CAst.make ~loc @@ QIntroInjection tc 
                                                   );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("<-")))))
                               (fun _ loc -> 
# 479 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc @@ QIntroRewrite false 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("->")))))
                               (fun _ loc -> 
# 478 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc @@ QIntroRewrite true 
                                             )])]}
        in let () =
        Pcoq.grammar_extend naming_intropattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm ident_or_anti)))
                                (fun id loc -> 
# 490 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroIdentifier id 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("?")))))
                               (fun _ loc -> 
# 488 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroAnonymous 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("?$")))))
                                               ((Pcoq.Symbol.nterm lident)))
                               (fun id _ loc -> 
# 486 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroFresh (QAnti id) 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PLEFTQMARK))))
                                               ((Pcoq.Symbol.nterm lident)))
                               (fun id _ loc -> 
# 484 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroFresh (QExpr id) 
                                                )])]}
        in let () =
        Pcoq.grammar_extend nonsimple_intropattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("**")))))
                                (fun _ loc -> 
# 496 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc @@ QIntroForthcoming false 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("*")))))
                               (fun _ loc -> 
# 495 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc @@ QIntroForthcoming true 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm simple_intropattern)))
                               (fun l loc -> 
# 494 "user-contrib/Ltac2/g_ltac2.mlg"
                                     l 
                                             )])]}
        in let () =
        Pcoq.grammar_extend simple_intropattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm simple_intropattern_closed)))
                                (fun pat loc -> 
# 502 "user-contrib/Ltac2/g_ltac2.mlg"
          pat 
                                                )])]}
        in let () =
        Pcoq.grammar_extend simple_intropattern_closed
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm naming_intropattern)))
                                (fun pat loc -> 
# 513 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroNaming pat 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("_")))))
                               (fun _ loc -> 
# 511 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroAction (CAst.make ~loc @@ QIntroWildcard) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm equality_intropattern)))
                               (fun pat loc -> 
# 509 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroAction pat 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm or_and_intropattern)))
                               (fun pat loc -> 
# 507 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QIntroAction (CAst.make ~loc @@ QIntroOrAndPattern pat) 
                                               )])]}
        in let () =
        Pcoq.grammar_extend q_intropatterns
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm intropatterns)))
                                (fun ipat loc -> 
# 517 "user-contrib/Ltac2/g_ltac2.mlg"
                                  ipat 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend q_intropattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm simple_intropattern)))
                                (fun ipat loc -> 
# 520 "user-contrib/Ltac2/g_ltac2.mlg"
                                        ipat 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend nat_or_anti
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("$")))))
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id _ loc -> 
# 524 "user-contrib/Ltac2/g_ltac2.mlg"
                                  QAnti (CAst.make ~loc id) 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm lnatural)))
                               (fun n loc -> 
# 523 "user-contrib/Ltac2/g_ltac2.mlg"
                          QExpr n 
                                             )])]}
        in let () =
        Pcoq.grammar_extend eqn_ipat
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 529 "user-contrib/Ltac2/g_ltac2.mlg"
             None 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("eqn"))))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                               ((Pcoq.Symbol.nterm naming_intropattern)))
                               (fun pat _ _ loc -> 
# 528 "user-contrib/Ltac2/g_ltac2.mlg"
                                                         Some pat 
                                                   )])]}
        in let () =
        Pcoq.grammar_extend with_bindings
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 533 "user-contrib/Ltac2/g_ltac2.mlg"
                                               CAst.make ~loc @@ QNoBindings 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                               ((Pcoq.Symbol.nterm bindings)))
                               (fun bl _ loc -> 
# 533 "user-contrib/Ltac2/g_ltac2.mlg"
                                   bl 
                                                )])]}
        in let () =
        Pcoq.grammar_extend constr_with_bindings
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Constr.constr)))
                                                ((Pcoq.Symbol.nterm with_bindings)))
                                (fun l c loc -> 
# 536 "user-contrib/Ltac2/g_ltac2.mlg"
                                                  CAst.make ~loc @@ (c, l) 
                                                )])]}
        in let () =
        Pcoq.grammar_extend destruction_arg
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm constr_with_bindings)))
                                (fun c loc -> 
# 541 "user-contrib/Ltac2/g_ltac2.mlg"
                                      CAst.make ~loc @@ QElimOnConstr c 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm lident)))
                               (fun id loc -> 
# 540 "user-contrib/Ltac2/g_ltac2.mlg"
                         CAst.make ~loc @@ QElimOnIdent id 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm lnatural)))
                               (fun n loc -> 
# 539 "user-contrib/Ltac2/g_ltac2.mlg"
                          CAst.make ~loc @@ QElimOnAnonHyp n 
                                             )])]}
        in let () =
        Pcoq.grammar_extend q_destruction_arg
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm destruction_arg)))
                                (fun arg loc -> 
# 545 "user-contrib/Ltac2/g_ltac2.mlg"
                                   arg 
                                                )])]}
        in let () =
        Pcoq.grammar_extend as_or_and_ipat
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 549 "user-contrib/Ltac2/g_ltac2.mlg"
             None 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("as")))))
                                               ((Pcoq.Symbol.nterm or_and_intropattern)))
                               (fun ipat _ loc -> 
# 548 "user-contrib/Ltac2/g_ltac2.mlg"
                                              Some ipat 
                                                  )])]}
        in let () =
        Pcoq.grammar_extend occs_nums
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("-")))))
                                                                ((Pcoq.Symbol.nterm nat_or_anti)))
                                                ((Pcoq.Symbol.list0 (Pcoq.Symbol.nterm nat_or_anti))))
                                (fun nl n _ loc -> 
# 555 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QAllOccurrencesBut (n::nl) 
                                                   );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm nat_or_anti)))))
                               (fun nl loc -> 
# 553 "user-contrib/Ltac2/g_ltac2.mlg"
                                    CAst.make ~loc @@ QOnlyOccurrences nl 
                                              )])]}
        in let () =
        Pcoq.grammar_extend occs
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 559 "user-contrib/Ltac2/g_ltac2.mlg"
                                                  CAst.make ~loc QAllOccurrences 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("at")))))
                                               ((Pcoq.Symbol.nterm occs_nums)))
                               (fun occs _ loc -> 
# 559 "user-contrib/Ltac2/g_ltac2.mlg"
                                    occs 
                                                  )])]}
        in let () =
        Pcoq.grammar_extend hypident
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("value"))))))
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("of"))))))
                                                                ((Pcoq.Symbol.nterm ident_or_anti)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                (fun _ id _ _ _ loc -> 
# 567 "user-contrib/Ltac2/g_ltac2.mlg"
          id,Locus.InHypValueOnly 
                                                       );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("type"))))))
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("of"))))))
                                                               ((Pcoq.Symbol.nterm ident_or_anti)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ id _ _ _ loc -> 
# 565 "user-contrib/Ltac2/g_ltac2.mlg"
          id,Locus.InHypTypeOnly 
                                                      );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm ident_or_anti)))
                               (fun id loc -> 
# 563 "user-contrib/Ltac2/g_ltac2.mlg"
          id,Locus.InHyp 
                                              )])]}
        in let () =
        Pcoq.grammar_extend hypident_occ
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm hypident)))
                                                ((Pcoq.Symbol.nterm occs)))
                                (fun occs h loc -> 
# 571 "user-contrib/Ltac2/g_ltac2.mlg"
                                   let (id,l) = h in ((occs,id),l) 
                                                   )])]}
        in let () =
        Pcoq.grammar_extend in_clause
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm hypident_occ)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                (fun hl loc -> 
# 581 "user-contrib/Ltac2/g_ltac2.mlg"
          { q_onhyps = Some hl; q_concl_occs = CAst.make ~loc QNoOccurrences } 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm hypident_occ)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                               ((Pcoq.Symbol.nterm concl_occ)))
                               (fun occs _ hl loc -> 
# 579 "user-contrib/Ltac2/g_ltac2.mlg"
          { q_onhyps = Some hl; q_concl_occs = occs } 
                                                     );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("*")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                               ((Pcoq.Symbol.nterm concl_occ)))
                               (fun occs _ _ loc -> 
# 577 "user-contrib/Ltac2/g_ltac2.mlg"
          { q_onhyps = None; q_concl_occs = occs } 
                                                    );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("*")))))
                                               ((Pcoq.Symbol.nterm occs)))
                               (fun occs _ loc -> 
# 575 "user-contrib/Ltac2/g_ltac2.mlg"
          { q_onhyps = None; q_concl_occs = occs } 
                                                  )])]}
        in let () =
        Pcoq.grammar_extend clause
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("at")))))
                                                ((Pcoq.Symbol.nterm occs_nums)))
                                (fun occs _ loc -> 
# 587 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ { q_onhyps = Some []; q_concl_occs = occs } 
                                                   );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("in")))))
                                               ((Pcoq.Symbol.nterm in_clause)))
                               (fun cl _ loc -> 
# 585 "user-contrib/Ltac2/g_ltac2.mlg"
                                  CAst.make ~loc @@ cl 
                                                )])]}
        in let () =
        Pcoq.grammar_extend q_clause
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm clause)))
                                (fun cl loc -> 
# 591 "user-contrib/Ltac2/g_ltac2.mlg"
                         cl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend concl_occ
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 595 "user-contrib/Ltac2/g_ltac2.mlg"
             CAst.make ~loc QNoOccurrences 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("*")))))
                                               ((Pcoq.Symbol.nterm occs)))
                               (fun occs _ loc -> 
# 594 "user-contrib/Ltac2/g_ltac2.mlg"
                              occs 
                                                  )])]}
        in let () =
        Pcoq.grammar_extend induction_clause
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm destruction_arg)))
                                                                ((Pcoq.Symbol.nterm as_or_and_ipat)))
                                                                ((Pcoq.Symbol.nterm eqn_ipat)))
                                                ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm clause))))
                                (fun cl eq pat c loc -> 
# 601 "user-contrib/Ltac2/g_ltac2.mlg"
        CAst.make ~loc @@ {
          indcl_arg = c;
          indcl_eqn = eq;
          indcl_as = pat;
          indcl_in = cl;
        } 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend q_induction_clause
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm induction_clause)))
                                (fun cl loc -> 
# 610 "user-contrib/Ltac2/g_ltac2.mlg"
                                   cl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend conversion
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Constr.constr)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("with")))))
                                                ((Pcoq.Symbol.nterm Constr.constr)))
                                (fun c2 _ c1 loc -> 
# 616 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QConvertWith (c1, c2) 
                                                    );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Constr.constr)))
                               (fun c loc -> 
# 614 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ QConvert c 
                                             )])]}
        in let () =
        Pcoq.grammar_extend q_conversion
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm conversion)))
                                (fun c loc -> 
# 620 "user-contrib/Ltac2/g_ltac2.mlg"
                            c 
                                              )])]}
        in let () =
        Pcoq.grammar_extend ltac2_orient
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 625 "user-contrib/Ltac2/g_ltac2.mlg"
             CAst.make ~loc None 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("<-")))))
                               (fun _ loc -> 
# 624 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc (Some false) 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("->")))))
                               (fun _ loc -> 
# 623 "user-contrib/Ltac2/g_ltac2.mlg"
                  CAst.make ~loc (Some true) 
                                             )])]}
        in let () =
        Pcoq.grammar_extend rewriter
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm constr_with_bindings)))
                                (fun c loc -> 
# 640 "user-contrib/Ltac2/g_ltac2.mlg"
          (CAst.make ~loc @@ QPrecisely (CAst.make 1), c) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm lnatural)))
                                               ((Pcoq.Symbol.nterm constr_with_bindings)))
                               (fun c n loc -> 
# 638 "user-contrib/Ltac2/g_ltac2.mlg"
          (CAst.make ~loc @@ QPrecisely n,c) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm lnatural)))
                                                               ((Pcoq.Symbol.rules 
                                                               [Pcoq.Rules.make 
                                                               (Pcoq.Rule.next_norec 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PLEFTQMARK))))
                                                               (fun _ loc ->
                                                               
# 635 "user-contrib/Ltac2/g_ltac2.mlg"
                                                      () 
                                                               );
                                                               Pcoq.Rules.make 
                                                               (Pcoq.Rule.next_norec 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("?")))))
                                                               (fun _ loc ->
                                                               
# 635 "user-contrib/Ltac2/g_ltac2.mlg"
                                () 
                                                               )])))
                                               ((Pcoq.Symbol.nterm constr_with_bindings)))
                               (fun c _ n loc -> 
# 636 "user-contrib/Ltac2/g_ltac2.mlg"
          (CAst.make ~loc @@ QUpTo n,c) 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm lnatural)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("!")))))
                                               ((Pcoq.Symbol.nterm constr_with_bindings)))
                               (fun c _ n loc -> 
# 634 "user-contrib/Ltac2/g_ltac2.mlg"
          (CAst.make ~loc @@ QPrecisely n,c) 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.rules 
                                                               [Pcoq.Rules.make 
                                                               (Pcoq.Rule.next_norec 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PLEFTQMARK))))
                                                               (fun _ loc ->
                                                               
# 631 "user-contrib/Ltac2/g_ltac2.mlg"
                                         () 
                                                               );
                                                               Pcoq.Rules.make 
                                                               (Pcoq.Rule.next_norec 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("?")))))
                                                               (fun _ loc ->
                                                               
# 631 "user-contrib/Ltac2/g_ltac2.mlg"
                   () 
                                                               )])))
                                               ((Pcoq.Symbol.nterm constr_with_bindings)))
                               (fun c _ loc -> 
# 632 "user-contrib/Ltac2/g_ltac2.mlg"
          (CAst.make ~loc @@ QRepeatStar,c) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("!")))))
                                               ((Pcoq.Symbol.nterm constr_with_bindings)))
                               (fun c _ loc -> 
# 630 "user-contrib/Ltac2/g_ltac2.mlg"
          (CAst.make ~loc @@ QRepeatPlus,c) 
                                               )])]}
        in let () =
        Pcoq.grammar_extend oriented_rewriter
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm ltac2_orient)))
                                                ((Pcoq.Symbol.nterm rewriter)))
                                (fun r b loc -> 
# 645 "user-contrib/Ltac2/g_ltac2.mlg"
        let (m, c) = r in
        CAst.make ~loc @@ {
        rew_orient = b;
        rew_repeat = m;
        rew_equatn = c;
      } 
                                                )])]}
        in let () =
        Pcoq.grammar_extend q_rewriting
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm oriented_rewriter)))
                                (fun r loc -> 
# 654 "user-contrib/Ltac2/g_ltac2.mlg"
                                   r 
                                              )])]}
        in let () =
        Pcoq.grammar_extend tactic_then_last
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 658 "user-contrib/Ltac2/g_ltac2.mlg"
             [] 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.opt (Pcoq.Symbol.nterml tac2expr ("6")))) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun lta _ loc -> 
# 657 "user-contrib/Ltac2/g_ltac2.mlg"
                                                               lta 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend tactic_then_gen
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 667 "user-contrib/Ltac2/g_ltac2.mlg"
             ([None], None) 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.nterm tactic_then_gen)))
                               (fun tg _ loc -> 
# 666 "user-contrib/Ltac2/g_ltac2.mlg"
                                       let (first,last) = tg in (None :: first, last) 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm tac2expr)))
                               (fun ta loc -> 
# 665 "user-contrib/Ltac2/g_ltac2.mlg"
                           ([Some ta], None) 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("..")))))
                                               ((Pcoq.Symbol.nterm tactic_then_last)))
                               (fun l _ loc -> 
# 664 "user-contrib/Ltac2/g_ltac2.mlg"
                                        ([], Some (None, l)) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("..")))))
                                               ((Pcoq.Symbol.nterm tactic_then_last)))
                               (fun l _ ta loc -> 
# 663 "user-contrib/Ltac2/g_ltac2.mlg"
                                                       ([], Some (Some ta, l)) 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm tac2expr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                               ((Pcoq.Symbol.nterm tactic_then_gen)))
                               (fun tg _ ta loc -> 
# 662 "user-contrib/Ltac2/g_ltac2.mlg"
                                                      let (first,last) = tg in (Some ta :: first, last) 
                                                   )])]}
        in let () =
        Pcoq.grammar_extend q_dispatch
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm tactic_then_gen)))
                                (fun d loc -> 
# 671 "user-contrib/Ltac2/g_ltac2.mlg"
                                 CAst.make ~loc d 
                                              )])]}
        in let () =
        Pcoq.grammar_extend q_occurrences
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm occs)))
                                (fun occs loc -> 
# 674 "user-contrib/Ltac2/g_ltac2.mlg"
                         occs 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend red_flag
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("delta"))))))
                                                ((Pcoq.Symbol.nterm delta_flag)))
                                (fun d _ loc -> 
# 683 "user-contrib/Ltac2/g_ltac2.mlg"
                                           d 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("zeta"))))))
                               (fun _ loc -> 
# 682 "user-contrib/Ltac2/g_ltac2.mlg"
                          CAst.make ~loc @@ QZeta 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("cofix"))))))
                               (fun _ loc -> 
# 681 "user-contrib/Ltac2/g_ltac2.mlg"
                           CAst.make ~loc @@ QCofix 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("fix"))))))
                               (fun _ loc -> 
# 680 "user-contrib/Ltac2/g_ltac2.mlg"
                         CAst.make ~loc @@ QFix 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("match"))))))
                               (fun _ loc -> 
# 679 "user-contrib/Ltac2/g_ltac2.mlg"
                           CAst.make ~loc @@ QMatch 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("iota"))))))
                               (fun _ loc -> 
# 678 "user-contrib/Ltac2/g_ltac2.mlg"
                          CAst.make ~loc @@ QIota 
                                             );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("beta"))))))
                               (fun _ loc -> 
# 677 "user-contrib/Ltac2/g_ltac2.mlg"
                          CAst.make ~loc @@ QBeta 
                                             )])]}
        in let () =
        Pcoq.grammar_extend refglobal
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("$")))))
                                                ((Pcoq.Symbol.nterm Prim.ident)))
                                (fun id _ loc -> 
# 689 "user-contrib/Ltac2/g_ltac2.mlg"
                                  QAnti (CAst.make ~loc id) 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.nterm Prim.qualid)))
                               (fun qid loc -> 
# 688 "user-contrib/Ltac2/g_ltac2.mlg"
                               QExpr (CAst.make ~loc @@ QReference qid) 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("&")))))
                                               ((Pcoq.Symbol.nterm Prim.ident)))
                               (fun id _ loc -> 
# 687 "user-contrib/Ltac2/g_ltac2.mlg"
                                  QExpr (CAst.make ~loc @@ QHypothesis id) 
                                                )])]}
        in let () =
        Pcoq.grammar_extend q_reference
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm refglobal)))
                                (fun r loc -> 
# 693 "user-contrib/Ltac2/g_ltac2.mlg"
                           r 
                                              )])]}
        in let () =
        Pcoq.grammar_extend refglobals
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm refglobal)))))
                                (fun gl loc -> 
# 696 "user-contrib/Ltac2/g_ltac2.mlg"
                                  CAst.make ~loc gl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend delta_flag
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 701 "user-contrib/Ltac2/g_ltac2.mlg"
             CAst.make ~loc @@ QDeltaBut (CAst.make ~loc []) 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.nterm refglobals)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ idl _ loc -> 
# 700 "user-contrib/Ltac2/g_ltac2.mlg"
                                        CAst.make ~loc @@ QConst idl 
                                                   );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("-")))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.nterm refglobals)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ idl _ _ loc -> 
# 699 "user-contrib/Ltac2/g_ltac2.mlg"
                                             CAst.make ~loc @@ QDeltaBut idl 
                                                     )])]}
        in let () =
        Pcoq.grammar_extend strategy_flag
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm delta_flag)))
                                (fun d loc -> 
# 707 "user-contrib/Ltac2/g_ltac2.mlg"
        CAst.make ~loc
          [CAst.make ~loc QBeta; CAst.make ~loc QIota; CAst.make ~loc QZeta; d] 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm red_flag)))))
                               (fun s loc -> 
# 705 "user-contrib/Ltac2/g_ltac2.mlg"
                                CAst.make ~loc s 
                                             )])]}
        in let () =
        Pcoq.grammar_extend q_strategy_flag
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm strategy_flag)))
                                (fun flag loc -> 
# 712 "user-contrib/Ltac2/g_ltac2.mlg"
                                  flag 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend hintdb
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.list1 ((Pcoq.Symbol.nterm ident_or_anti)))))
                                (fun l loc -> 
# 716 "user-contrib/Ltac2/g_ltac2.mlg"
                                     CAst.make ~loc @@ QHintDbs l 
                                              );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("*")))))
                               (fun _ loc -> 
# 715 "user-contrib/Ltac2/g_ltac2.mlg"
                 CAst.make ~loc @@ QHintAll 
                                             )])]}
        in let () =
        Pcoq.grammar_extend q_hintdb
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm hintdb)))
                                (fun db loc -> 
# 720 "user-contrib/Ltac2/g_ltac2.mlg"
                         db 
                                               )])]}
        in let () =
        Pcoq.grammar_extend match_pattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm Constr.lconstr_pattern)))
                                (fun pat loc -> 
# 725 "user-contrib/Ltac2/g_ltac2.mlg"
                                          CAst.make ~loc @@ QConstrMatchPattern pat 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("context"))))))
                                                               ((Pcoq.Symbol.opt (Pcoq.Symbol.nterm Prim.ident))))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr_pattern)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                               (fun _ pat _ id _ loc -> 
# 724 "user-contrib/Ltac2/g_ltac2.mlg"
                                                      CAst.make ~loc @@ QConstrMatchContext (id, pat) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend match_rule
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm match_pattern)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                                ((Pcoq.Symbol.nterm tac2expr)))
                                (fun tac _ mp loc -> 
# 729 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ (mp, tac) 
                                                     )])]}
        in let () =
        Pcoq.grammar_extend match_list
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm match_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                (fun mrl _ loc -> 
# 734 "user-contrib/Ltac2/g_ltac2.mlg"
                                                 CAst.make ~loc @@ mrl 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm match_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun mrl loc -> 
# 733 "user-contrib/Ltac2/g_ltac2.mlg"
                                            CAst.make ~loc @@ mrl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend q_constr_matching
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm match_list)))
                                (fun m loc -> 
# 737 "user-contrib/Ltac2/g_ltac2.mlg"
                            m 
                                              )])]}
        in let () =
        Pcoq.grammar_extend gmatch_hyp_pattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Prim.name)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                ((Pcoq.Symbol.nterm match_pattern)))
                                (fun pat _ na loc -> 
# 740 "user-contrib/Ltac2/g_ltac2.mlg"
                                                      (na, pat) 
                                                     )])]}
        in let () =
        Pcoq.grammar_extend gmatch_pattern
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("[")))))
                                                                ((Pcoq.Symbol.list0sep ((Pcoq.Symbol.nterm gmatch_hyp_pattern)) ((Pcoq.Symbol.token (Tok.PKEYWORD (",")))) false)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("|-")))))
                                                                ((Pcoq.Symbol.nterm match_pattern)))
                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("]")))))
                                (fun _ p _ hl _ loc -> 
# 744 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ {
          q_goal_match_concl = p;
          q_goal_match_hyps = hl;
        } 
                                                       )])]}
        in let () =
        Pcoq.grammar_extend gmatch_rule
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm gmatch_pattern)))
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("=>")))))
                                                ((Pcoq.Symbol.nterm tac2expr)))
                                (fun tac _ mp loc -> 
# 752 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc @@ (mp, tac) 
                                                     )])]}
        in let () =
        Pcoq.grammar_extend gmatch_list
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))))
                                                ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm gmatch_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                                (fun mrl _ loc -> 
# 757 "user-contrib/Ltac2/g_ltac2.mlg"
                                                  CAst.make ~loc @@ mrl 
                                                  );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.stop)
                                               ((Pcoq.Symbol.list1sep ((Pcoq.Symbol.nterm gmatch_rule)) ((Pcoq.Symbol.token (Tok.PKEYWORD ("|")))) false)))
                               (fun mrl loc -> 
# 756 "user-contrib/Ltac2/g_ltac2.mlg"
                                             CAst.make ~loc @@ mrl 
                                               )])]}
        in let () =
        Pcoq.grammar_extend q_goal_matching
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm gmatch_list)))
                                (fun m loc -> 
# 760 "user-contrib/Ltac2/g_ltac2.mlg"
                             m 
                                              )])]}
        in let () =
        Pcoq.grammar_extend move_location
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                                ("before"))))))
                                                ((Pcoq.Symbol.nterm ident_or_anti)))
                                (fun id _ loc -> 
# 766 "user-contrib/Ltac2/g_ltac2.mlg"
                                                CAst.make ~loc @@ QMoveBefore id 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                                               ("after"))))))
                                               ((Pcoq.Symbol.nterm ident_or_anti)))
                               (fun id _ loc -> 
# 765 "user-contrib/Ltac2/g_ltac2.mlg"
                                               CAst.make ~loc @@ QMoveAfter id 
                                                );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("at")))))
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("bottom"))))))
                               (fun _ _ loc -> 
# 764 "user-contrib/Ltac2/g_ltac2.mlg"
                                  CAst.make ~loc @@ QMoveLast 
                                               );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("at")))))
                                               ((Pcoq.Symbol.token (Tok.PIDENT (Some
                                               ("top"))))))
                               (fun _ _ loc -> 
# 763 "user-contrib/Ltac2/g_ltac2.mlg"
                               CAst.make ~loc @@ QMoveFirst 
                                               )])]}
        in let () =
        Pcoq.grammar_extend q_move_location
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm move_location)))
                                (fun mv loc -> 
# 770 "user-contrib/Ltac2/g_ltac2.mlg"
                                mv 
                                               )])]}
        in let () =
        Pcoq.grammar_extend as_name
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.token (Tok.PKEYWORD ("as")))))
                                                ((Pcoq.Symbol.nterm ident_or_anti)))
                                (fun id _ loc -> 
# 774 "user-contrib/Ltac2/g_ltac2.mlg"
                                      Some id 
                                                 );
                               Pcoq.Production.make (Pcoq.Rule.stop)
                               (fun loc -> 
# 773 "user-contrib/Ltac2/g_ltac2.mlg"
             None 
                                           )])]}
        in let () =
        Pcoq.grammar_extend pose
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Constr.constr)))
                                                ((Pcoq.Symbol.nterm as_name)))
                                (fun na c loc -> 
# 780 "user-contrib/Ltac2/g_ltac2.mlg"
                                             CAst.make ~loc (na, c) 
                                                 );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm test_lpar_id_coloneq)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm ident_or_anti)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ c _ id _ _ loc -> 
# 779 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc (Some id, c) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend q_pose
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm pose)))
                                (fun p loc -> 
# 784 "user-contrib/Ltac2/g_ltac2.mlg"
                      p 
                                              )])]}
        in let () =
        Pcoq.grammar_extend as_ipat
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 788 "user-contrib/Ltac2/g_ltac2.mlg"
             None 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("as")))))
                                               ((Pcoq.Symbol.nterm simple_intropattern)))
                               (fun ipat _ loc -> 
# 787 "user-contrib/Ltac2/g_ltac2.mlg"
                                              Some ipat 
                                                  )])]}
        in let () =
        Pcoq.grammar_extend by_tactic
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make (Pcoq.Rule.stop)
                                (fun loc -> 
# 793 "user-contrib/Ltac2/g_ltac2.mlg"
             None 
                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("by")))))
                                               ((Pcoq.Symbol.nterm tac2expr)))
                               (fun tac _ loc -> 
# 792 "user-contrib/Ltac2/g_ltac2.mlg"
                                  Some tac 
                                                 )])]}
        in let () =
        Pcoq.grammar_extend assertion
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                                (Pcoq.Rule.stop)
                                                                ((Pcoq.Symbol.nterm Constr.constr)))
                                                                ((Pcoq.Symbol.nterm as_ipat)))
                                                ((Pcoq.Symbol.nterm by_tactic)))
                                (fun tac ipat c loc -> 
# 803 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc (QAssertType (ipat, c, tac)) 
                                                       );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm test_lpar_id_colon)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm ident_or_anti)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                                               ((Pcoq.Symbol.nterm by_tactic)))
                               (fun tac _ c _ id _ _ loc -> 
# 800 "user-contrib/Ltac2/g_ltac2.mlg"
        let ipat = CAst.make ~loc @@ QIntroNaming (CAst.make ~loc @@ QIntroIdentifier id) in
        CAst.make ~loc (QAssertType (Some ipat, c, tac)) 
                                                            );
                               Pcoq.Production.make
                               (Pcoq.Rule.next (Pcoq.Rule.next (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.next 
                                                               (Pcoq.Rule.stop)
                                                               ((Pcoq.Symbol.nterm test_lpar_id_coloneq)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD ("(")))))
                                                               ((Pcoq.Symbol.nterm ident_or_anti)))
                                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (":=")))))
                                                               ((Pcoq.Symbol.nterm Constr.lconstr)))
                                               ((Pcoq.Symbol.token (Tok.PKEYWORD (")")))))
                               (fun _ c _ id _ _ loc -> 
# 798 "user-contrib/Ltac2/g_ltac2.mlg"
          CAst.make ~loc (QAssertValue (id, c)) 
                                                        )])]}
        in let () =
        Pcoq.grammar_extend q_assert
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm assertion)))
                                (fun a loc -> 
# 807 "user-contrib/Ltac2/g_ltac2.mlg"
                           a 
                                              )])]}
        in ()


# 831 "user-contrib/Ltac2/g_ltac2.mlg"
 

let () =

let open Tok in
let (++) r s = Pcoq.Rule.next r s in
let rules = [
  Pcoq.(
    Production.make
      (Rule.stop ++ Symbol.nterm test_dollar_ident ++ Symbol.token (PKEYWORD "$") ++ Symbol.nterm Prim.ident)
    begin fun id _ _ loc ->
      let id = Loc.tag ~loc id in
      let arg = Genarg.in_gen (Genarg.rawwit Tac2env.wit_ltac2_quotation) id in
      CAst.make ~loc (CHole (None, Namegen.IntroAnonymous, Some arg))
    end
  );

  Pcoq.(
    Production.make
      (Rule.stop ++ Symbol.nterm test_ampersand_ident ++ Symbol.token (PKEYWORD "&") ++ Symbol.nterm Prim.ident)
    begin fun id _ _ loc ->
      let tac = Tac2quote.of_exact_hyp ~loc (CAst.make ~loc id) in
      let arg = Genarg.in_gen (Genarg.rawwit Tac2env.wit_ltac2_constr) tac in
      CAst.make ~loc (CHole (None, Namegen.IntroAnonymous, Some arg))
    end
  );

  Pcoq.(
    Production.make
      (Rule.stop ++ Symbol.token (PIDENT (Some "ltac2")) ++ Symbol.token (PKEYWORD ":") ++
       Symbol.token (PKEYWORD "(") ++ Symbol.nterm tac2expr ++ Symbol.token (PKEYWORD ")"))
    begin fun _ tac _ _ _ loc ->
      let arg = Genarg.in_gen (Genarg.rawwit Tac2env.wit_ltac2_constr) tac in
      CAst.make ~loc (CHole (None, Namegen.IntroAnonymous, Some arg))
    end
  )
] in

Hook.set Tac2entries.register_constr_quotations begin fun () ->
  Pcoq.grammar_extend Pcoq.Constr.operconstr {pos=Some (Gramlib.Gramext.Level "0"); data=[(None, None, rules)]}
end




# 875 "user-contrib/Ltac2/g_ltac2.mlg"
 

let pr_ltac2entry _ = mt () (* FIXME *)
let pr_ltac2expr _ = mt () (* FIXME *)



let (wit_ltac2_entry, ltac2_entry) = Vernacextend.vernac_argument_extend ~name:"ltac2_entry" 
                                     {
                                     Vernacextend.arg_parsing = Vernacextend.Arg_rules (
                                                                [(Pcoq.Production.make
                                                                  (Pcoq.Rule.next 
                                                                  (Pcoq.Rule.stop)
                                                                  ((Pcoq.Symbol.nterm tac2def_mut)))
                                                                  (fun e
                                                                  loc -> 
                                                                  
# 888 "user-contrib/Ltac2/g_ltac2.mlg"
                          e 
                                                                  ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm tac2def_syn)))
                                                                 (fun e
                                                                 loc -> 
                                                                 
# 887 "user-contrib/Ltac2/g_ltac2.mlg"
                          e 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm tac2def_ext)))
                                                                 (fun e
                                                                 loc -> 
                                                                 
# 886 "user-contrib/Ltac2/g_ltac2.mlg"
                          e 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm tac2def_typ)))
                                                                 (fun t
                                                                 loc -> 
                                                                 
# 885 "user-contrib/Ltac2/g_ltac2.mlg"
                          t 
                                                                 ));
                                                                (Pcoq.Production.make
                                                                 (Pcoq.Rule.next 
                                                                 (Pcoq.Rule.stop)
                                                                 ((Pcoq.Symbol.nterm tac2def_val)))
                                                                 (fun v
                                                                 loc -> 
                                                                 
# 884 "user-contrib/Ltac2/g_ltac2.mlg"
                          v 
                                                                 ))]);
                                     Vernacextend.arg_printer = fun env sigma -> 
                                     
# 883 "user-contrib/Ltac2/g_ltac2.mlg"
             pr_ltac2entry 
                                     ;
                                     }
let _ = (wit_ltac2_entry, ltac2_entry)

let (wit_ltac2_expr, ltac2_expr) = Vernacextend.vernac_argument_extend ~name:"ltac2_expr" 
                                   {
                                   Vernacextend.arg_parsing = Vernacextend.Arg_alias (tac2expr);
                                   Vernacextend.arg_printer = fun env sigma -> 
                                   
# 892 "user-contrib/Ltac2/g_ltac2.mlg"
             pr_ltac2expr 
                                   ;
                                   }
let _ = (wit_ltac2_expr, ltac2_expr)


# 896 "user-contrib/Ltac2/g_ltac2.mlg"
 

let classify_ltac2 = function
| StrSyn _ -> Vernacextend.(VtSideff ([], VtNow))
| StrMut _ | StrVal _ | StrPrm _  | StrTyp _ -> Vernacextend.classify_as_sideeff



let () = Vernacextend.vernac_extend ~command:"VernacDeclareTactic2Definition"  ?entry:None 
         [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Ltac2", 
                                     Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_ltac2_entry), 
                                     Vernacextend.TyNil)), (let coqpp_body e
                                                           local = Vernacextend.VtDefault (fun () -> 
                                                                   
# 905 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                               
  Tac2entries.register_struct ?local e
  
                                                                   ) in fun e
                                                           ~atts
                                                           -> coqpp_body e
                                                           (Attributes.parse locality atts)), Some 
         (fun e -> 
# 905 "user-contrib/Ltac2/g_ltac2.mlg"
                                                        classify_ltac2 e 
         )));
         (Vernacextend.TyML (false, Vernacextend.TyTerminal ("Ltac2", 
                                    Vernacextend.TyTerminal ("Eval", 
                                    Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_ltac2_expr), 
                                    Vernacextend.TyNil))), (let coqpp_body e
                                                           () = Vernacextend.VtReadProofOpt (
                                                                
# 908 "user-contrib/Ltac2/g_ltac2.mlg"
                                                                                                  
  fun ~pstate -> Tac2entries.perform_eval ~pstate e
  
                                                                ) in fun e
                                                           ~atts
                                                           -> coqpp_body e
                                                           (Attributes.unsupported_attributes atts)), Some 
         (fun e -> 
# 908 "user-contrib/Ltac2/g_ltac2.mlg"
                                                           Vernacextend.classify_as_sideeff 
         )))]


# 913 "user-contrib/Ltac2/g_ltac2.mlg"
 

let _ = Pvernac.register_proof_mode "Ltac2" tac2mode

open G_ltac
open Vernacextend



let () = Vernacextend.vernac_extend ~command:"VernacLtac2"  ?entry:(Some ( tac2mode )) 
         [(Vernacextend.TyML (false, Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_ltac2_expr), 
                                     Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_ltac_use_default), 
                                     Vernacextend.TyNil)), (let coqpp_body t
                                                           default
                                                           () = Vernacextend.VtModifyProof (
                                                                
# 924 "user-contrib/Ltac2/g_ltac2.mlg"
                                  
(*     let g = Option.default (Proof_global.get_default_goal_selector ()) g in *)
  fun ~pstate -> Tac2entries.call ~pstate ~default t 
                                                                ) in fun t
                                                           default ~atts
                                                           -> coqpp_body t
                                                           default
                                                           (Attributes.unsupported_attributes atts)), Some 
         (fun t default -> 
# 924 "user-contrib/Ltac2/g_ltac2.mlg"
      classify_as_proofstep 
         )))]

let _ = let () =
        Pcoq.grammar_extend tac2mode
        { Pcoq.pos=None; data=[(None, None,
                               [Pcoq.Production.make
                                (Pcoq.Rule.next (Pcoq.Rule.stop)
                                                ((Pcoq.Symbol.nterm G_vernac.query_command)))
                                (fun tac loc -> 
# 932 "user-contrib/Ltac2/g_ltac2.mlg"
                                          tac None 
                                                )])]}
        in ()


# 935 "user-contrib/Ltac2/g_ltac2.mlg"
 

open Stdarg



let () = Vernacextend.vernac_extend ~command:"Ltac2Print" ~classifier:(fun _ -> Vernacextend.classify_as_sideeff) ?entry:None 
         [(Vernacextend.TyML (false, Vernacextend.TyTerminal ("Print", 
                                     Vernacextend.TyTerminal ("Ltac2", 
                                     Vernacextend.TyNonTerminal (Extend.TUentry (Genarg.get_arg_tag wit_reference), 
                                     Vernacextend.TyNil))), (let coqpp_body tac
                                                            () = Vernacextend.VtDefault (fun () -> 
                                                                 
# 942 "user-contrib/Ltac2/g_ltac2.mlg"
                                          Tac2entries.print_ltac tac 
                                                                 ) in fun tac
                                                            ~atts
                                                            -> coqpp_body tac
                                                            (Attributes.unsupported_attributes atts)), None))]

